# coding: utf-8

# Python libs
from __future__ import absolute_import

import datetime
import logging
from uuid import UUID

# Salt libs
import salt.beacons.journald as journald
import salt.utils.data
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import Mock

# Salt testing libs
from tests.support.unit import TestCase

log = logging.getLogger(__name__)

_STUB_JOURNALD_ENTRY = {
    "_BOOT_ID": UUID("ad3915a5-9008-4fec-a635-140606525497"),
    "__MONOTONIC_TIMESTAMP": (
        datetime.timedelta(4, 28586, 703069),
        UUID("ad3915a5-9008-4fec-a635-140606525497"),
    ),
    "_AUDIT_LOGINUID": 1000,
    "SYSLOG_FACILITY": 10,
    "_SYSTEMD_SLICE": u"system.slice",
    "_GID": 0,
    "__REALTIME_TIMESTAMP": datetime.datetime(2017, 6, 27, 20, 8, 16, 468468),
    "_AUDIT_SESSION": 351,
    "PRIORITY": 6,
    "_TRANSPORT": u"syslog",
    "_HOSTNAME": u"hostname",
    "_CAP_EFFECTIVE": u"3fffffffff",
    "_SYSTEMD_UNIT": u"ssh.service",
    "_MACHINE_ID": UUID("14fab5bb-228d-414b-bdf4-cbc62cb7ba54"),
    "_PID": 15091,
    "SYSLOG_IDENTIFIER": u"sshd",
    "_SOURCE_REALTIME_TIMESTAMP": datetime.datetime(2017, 6, 27, 20, 8, 16, 468454),
    "_SYSTEMD_CGROUP": u"/system.slice/ssh.service",
    "__CURSOR": "s=7711ee01b03446309383870171dd5839;i=a74e;b=ad3915a590084feca635140606525497;m=571f43f8 dd;t=552fc7ed1cdf4;x=4ca0a3d4f1905736",
    "_COMM": u"sshd",
    "_CMDLINE": u"sshd: gareth [priv]",
    "_SYSTEMD_INVOCATION_ID": u"38a5d5aad292426d93bfaab72a69c2ab",
    "_EXE": u"/usr/sbin/sshd",
    "_UID": 0,
    "SYSLOG_PID": 15091,
    "MESSAGE": u"pam_unix(sshd:session): session opened for user username by (uid=0)",
}


class SystemdJournaldMock(Mock):
    """ Request Mock"""

    returned_once = False

    def get_next(self, *args, **kwargs):
        if not self.returned_once:
            self.returned_once = True
            return _STUB_JOURNALD_ENTRY
        else:
            return None

    def seek_tail(self, *args, **kwargs):
        return {}

    def get_previous(self, *args, **kwargs):
        return {}


SYSTEMD_MOCK = SystemdJournaldMock()


class JournaldBeaconTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test case for salt.beacons.journald
    """

    def setup_loader_modules(self):
        return {
            journald: {
                "__context__": {"systemd.journald": SYSTEMD_MOCK},
                "__salt__": {},
            }
        }

    def test_non_list_config(self):
        config = {}

        ret = journald.validate(config)

        self.assertEqual(
            ret, (False, "Configuration for journald beacon must be a list.")
        )

    def test_empty_config(self):
        config = [{}]

        ret = journald.validate(config)

        self.assertEqual(ret, (True, "Valid beacon configuration"))

    def test_journald_match(self):
        config = [{"services": {"sshd": {"SYSLOG_IDENTIFIER": "sshd", "PRIORITY": 6}}}]

        ret = journald.validate(config)

        self.assertEqual(ret, (True, "Valid beacon configuration"))

        _expected_return = salt.utils.data.simple_types_filter(_STUB_JOURNALD_ENTRY)
        _expected_return["tag"] = "sshd"

        ret = journald.beacon(config)
        self.assertEqual(ret, [_expected_return])



# -*- coding: utf-8 -*-
"""
    :codeauthor: Logilab <contact@logilab.fr>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.postgres_cluster as postgres_cluster

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class PostgresClusterTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.postgres_cluster
    """

    def setup_loader_modules(self):
        return {postgres_cluster: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to ensure that the named database is present
        with the specified properties.
        """
        name = "main"
        version = "9.4"

        ret = {"name": name, "changes": {}, "result": False, "comment": ""}

        mock_t = MagicMock(return_value=True)
        mock_f = MagicMock(return_value=False)
        infos = {"{0}/{1}".format(version, name): {}}
        mock = MagicMock(return_value=infos)
        with patch.dict(
            postgres_cluster.__salt__,
            {
                "postgres.cluster_list": mock,
                "postgres.cluster_exists": mock_t,
                "postgres.cluster_create": mock_t,
            },
        ):
            comt = "Cluster {0}/{1} is already present".format(version, name)
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(postgres_cluster.present(version, name), ret)
            infos["{0}/{1}".format(version, name)]["port"] = 5433
            comt = (
                "Cluster {0}/{1} has wrong parameters "
                "which couldn't be changed on fly.".format(version, name)
            )
            ret.update({"comment": comt, "result": False})
            self.assertDictEqual(
                postgres_cluster.present(version, name, port=5434), ret
            )
            infos["{0}/{1}".format(version, name)]["datadir"] = "/tmp/"
            comt = (
                "Cluster {0}/{1} has wrong parameters "
                "which couldn't be changed on fly.".format(version, name)
            )
            ret.update({"comment": comt, "result": False})
            self.assertDictEqual(
                postgres_cluster.present(version, name, port=5434), ret
            )

        with patch.dict(
            postgres_cluster.__salt__,
            {
                "postgres.cluster_list": mock,
                "postgres.cluster_exists": mock_f,
                "postgres.cluster_create": mock_t,
            },
        ):
            comt = "The cluster {0}/{1} has been created".format(version, name)
            ret.update(
                {
                    "comment": comt,
                    "result": True,
                    "changes": {"{0}/{1}".format(version, name): "Present"},
                }
            )
            self.assertDictEqual(postgres_cluster.present(version, name), ret)
            with patch.dict(postgres_cluster.__opts__, {"test": True}):
                comt = "Cluster {0}/{1} is set to be created".format(version, name)
                ret.update({"comment": comt, "result": None, "changes": {}})
                self.assertDictEqual(postgres_cluster.present(version, name), ret)

        with patch.dict(
            postgres_cluster.__salt__,
            {
                "postgres.cluster_list": mock,
                "postgres.cluster_exists": mock_f,
                "postgres.cluster_create": mock_f,
            },
        ):
            comt = "Failed to create cluster {0}/{1}".format(version, name)
            ret.update({"comment": comt, "result": False})
            self.assertDictEqual(postgres_cluster.present(version, name), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to ensure that the named database is absent.
        """
        name = "main"
        version = "9.4"

        ret = {"name": name, "changes": {}, "result": False, "comment": ""}

        mock_t = MagicMock(return_value=True)
        mock = MagicMock(side_effect=[True, True, False])
        with patch.dict(
            postgres_cluster.__salt__,
            {"postgres.cluster_exists": mock, "postgres.cluster_remove": mock_t},
        ):
            with patch.dict(postgres_cluster.__opts__, {"test": True}):
                comt = "Cluster {0}/{1} is set to be removed".format(version, name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(postgres_cluster.absent(version, name), ret)

            with patch.dict(postgres_cluster.__opts__, {"test": False}):
                comt = "Cluster {0}/{1} has been removed".format(version, name)
                ret.update(
                    {"comment": comt, "result": True, "changes": {name: "Absent"}}
                )
                self.assertDictEqual(postgres_cluster.absent(version, name), ret)

                comt = "Cluster {0}/{1} is not present, so it cannot be removed".format(
                    version, name
                )
                ret.update({"comment": comt, "result": True, "changes": {}})
                self.assertDictEqual(postgres_cluster.absent(version, name), ret)



# coding: utf-8
from __future__ import absolute_import, print_function, unicode_literals

import os
import shutil
import tempfile
import time

import salt.utils.files
import salt.utils.platform
from salt.beacons import watchdog
from salt.ext.six.moves import range
from tests.support.helpers import slowTest
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf


def check_events(config):
    total_delay = 1
    delay_per_loop = 20e-3

    for _ in range(int(total_delay / delay_per_loop)):
        events = watchdog.beacon(config)

        if events:
            return events

        time.sleep(delay_per_loop)

    return []


def create(path, content=None):
    with salt.utils.files.fopen(path, "w") as f:
        if content:
            f.write(content)
        os.fsync(f)


@skipIf(not watchdog.HAS_WATCHDOG, "watchdog is not available")
@skipIf(
    salt.utils.platform.is_darwin(),
    "Tests were being skipped pre macos under nox. Keep it like that for now.",
)
class IWatchdogBeaconTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test case for salt.beacons.watchdog
    """

    def setup_loader_modules(self):
        return {watchdog: {}}

    def setUp(self):
        self.tmpdir = tempfile.mkdtemp()

    def tearDown(self):
        watchdog.close({})
        shutil.rmtree(self.tmpdir, ignore_errors=True)

    def assertValid(self, config):
        ret = watchdog.validate(config)
        self.assertEqual(ret, (True, "Valid beacon configuration"))

    def test_empty_config(self):
        config = [{}]
        ret = watchdog.beacon(config)
        self.assertEqual(ret, [])

    def test_file_create(self):
        path = os.path.join(self.tmpdir, "tmpfile")

        config = [{"directories": {self.tmpdir: {"mask": ["create"]}}}]
        self.assertValid(config)
        self.assertEqual(watchdog.beacon(config), [])

        create(path)

        ret = check_events(config)
        self.assertEqual(len(ret), 1)
        self.assertEqual(ret[0]["path"], path)
        self.assertEqual(ret[0]["change"], "created")

    def test_file_modified(self):
        path = os.path.join(self.tmpdir, "tmpfile")
        # Create triggers a modify event along with the create event in Py3
        # So, let's do this before configuring the beacon
        create(path)

        config = [{"directories": {self.tmpdir: {"mask": ["modify"]}}}]
        self.assertValid(config)
        self.assertEqual(watchdog.beacon(config), [])

        create(path, "some content")

        ret = check_events(config)

        modified = False
        for event in ret:
            # "modified" requires special handling
            # A modification sometimes triggers 2 modified events depending on
            # the OS and the python version
            # When the "modified" event triggers on modify, it will have the
            # path to the temp file (path), other modified events will contain
            # the path minus "tmpfile" and will not match. That's how we'll
            # distinguish the two
            if event["change"] == "modified":
                if event["path"] == path:
                    modified = True

        # Check results of the for loop to validate modified
        self.assertTrue(modified)

    def test_file_deleted(self):
        path = os.path.join(self.tmpdir, "tmpfile")
        create(path)

        config = [{"directories": {self.tmpdir: {"mask": ["delete"]}}}]
        self.assertValid(config)
        self.assertEqual(watchdog.beacon(config), [])

        os.remove(path)

        ret = check_events(config)
        self.assertEqual(len(ret), 1)
        self.assertEqual(ret[0]["path"], path)
        self.assertEqual(ret[0]["change"], "deleted")

    def test_file_moved(self):
        path = os.path.join(self.tmpdir, "tmpfile")
        create(path)

        config = [{"directories": {self.tmpdir: {"mask": ["move"]}}}]
        self.assertValid(config)
        self.assertEqual(watchdog.beacon(config), [])

        os.rename(path, path + "_moved")

        ret = check_events(config)
        self.assertEqual(len(ret), 1)
        self.assertEqual(ret[0]["path"], path)
        self.assertEqual(ret[0]["change"], "moved")

    def test_file_create_in_directory(self):
        config = [{"directories": {self.tmpdir: {"mask": ["create"]}}}]
        self.assertValid(config)
        self.assertEqual(watchdog.beacon(config), [])

        path = os.path.join(self.tmpdir, "tmpfile")
        create(path)

        ret = check_events(config)
        self.assertEqual(len(ret), 1)
        self.assertEqual(ret[0]["path"], path)
        self.assertEqual(ret[0]["change"], "created")

    @slowTest
    def test_trigger_all_possible_events(self):
        path = os.path.join(self.tmpdir, "tmpfile")
        moved = path + "_moved"

        config = [{"directories": {self.tmpdir: {}}}]
        self.assertValid(config)
        self.assertEqual(watchdog.beacon(config), [])

        # create
        create(path)
        # modify
        create(path, "modified content")
        # move
        os.rename(path, moved)
        # delete
        os.remove(moved)

        # Give the events time to load into the queue
        time.sleep(1)

        ret = check_events(config)

        events = {"created": "", "deleted": "", "moved": ""}
        modified = False
        for event in ret:
            if event["change"] == "created":
                self.assertEqual(event["path"], path)
                events.pop("created", "")
            if event["change"] == "moved":
                self.assertEqual(event["path"], path)
                events.pop("moved", "")
            if event["change"] == "deleted":
                self.assertEqual(event["path"], moved)
                events.pop("deleted", "")
            # "modified" requires special handling
            # All events [created, moved, deleted] also trigger a "modified"
            # event on Linux
            # Only the "created" event triggers a modified event on Py3 Windows
            # When the "modified" event triggers on modify, it will have the
            # path to the temp file (path), other modified events will contain
            # the path minus "tmpfile" and will not match. That's how we'll
            # distinguish the two
            if event["change"] == "modified":
                if event["path"] == path:
                    modified = True

        # Check results of the for loop to validate modified
        self.assertTrue(modified)

        # Make sure all events were checked
        self.assertDictEqual(events, {})



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import salt libs
import salt.modules.postgres as postgresmod
import salt.states.postgres_database as postgres_database
import salt.states.postgres_extension as postgres_extension
import salt.states.postgres_group as postgres_group
import salt.states.postgres_schema as postgres_schema
import salt.states.postgres_user as postgres_user

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, Mock, patch
from tests.support.unit import TestCase


class PostgresUserTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        patcher = patch("salt.utils.path.which", Mock(return_value="/usr/bin/pgsql"))
        patcher.start()
        self.addCleanup(patcher.stop)
        self.salt_stub = {
            "config.option": Mock(),
            "cmd.run_all": Mock(),
            "file.chown": Mock(),
            "file.remove": Mock(),
        }
        self.addCleanup(delattr, self, "salt_stub")
        return {
            postgres_database: {},
            postgres_group: {},
            postgres_extension: {},
            postgres_schema: {},
            postgres_user: {
                "__grains__": {"os_family": "Linux"},
                "__salt__": self.salt_stub,
                "__opts__": {"test": False},
            },
        }

    def test_present__creation(self):
        # test=True
        with patch.dict(
            postgres_user.__salt__,
            {
                "postgres.role_get": Mock(return_value=None),
                "postgres.user_create": MagicMock(),
            },
        ):
            with patch.dict(postgres_user.__opts__, {"test": True}):
                ret = postgres_user.present("foo")
                self.assertEqual(
                    ret,
                    {
                        "comment": "User foo is set to be created",
                        "changes": {},
                        "name": "foo",
                        "result": None,
                    },
                )
                self.assertEqual(self.salt_stub["postgres.user_create"].call_count, 0)

            # test=False
            ret = postgres_user.present("foo")
            self.assertEqual(
                ret,
                {
                    "comment": "The user foo has been created",
                    "changes": {"foo": "Present"},
                    "name": "foo",
                    "result": True,
                },
            )
            self.salt_stub["postgres.user_create"].assert_called_once_with(
                username="foo",
                superuser=None,
                encrypted=True,
                runas=None,
                inherit=None,
                rolepassword=None,
                port=None,
                replication=None,
                host=None,
                createroles=None,
                user=None,
                groups=None,
                maintenance_db=None,
                login=None,
                password=None,
                valid_until=None,
                createdb=None,
            )

    def test_present__update(self):
        # test=True
        with patch.dict(
            postgres_user.__salt__,
            {
                "postgres.role_get": Mock(
                    return_value={
                        "can create databases": False,
                        "can create roles": False,
                        "can login": False,
                        "can update system catalogs": False,
                        "connections": None,
                        "defaults variables": {},
                        "expiry time": None,
                        "inherits privileges": True,
                        "replication": False,
                        "superuser": False,
                    }
                ),
                "postgres.user_update": MagicMock(),
            },
        ):
            with patch.dict(postgres_user.__opts__, {"test": True}):
                ret = postgres_user.present("foo", login=True, replication=False)
                self.assertEqual(
                    ret,
                    {
                        "comment": "User foo is set to be updated",
                        "changes": {"foo": {"login": True}},
                        "name": "foo",
                        "result": None,
                    },
                )
                self.assertEqual(self.salt_stub["postgres.user_update"].call_count, 0)

            # test=False
            ret = postgres_user.present("foo", login=True, replication=False)
            self.assertEqual(
                ret,
                {
                    "comment": "The user foo has been updated",
                    "changes": {"foo": {"login": True}},
                    "name": "foo",
                    "result": True,
                },
            )
            self.salt_stub["postgres.user_update"].assert_called_once_with(
                username="foo",
                superuser=None,
                encrypted=True,
                runas=None,
                inherit=None,
                rolepassword=None,
                port=None,
                replication=False,
                host=None,
                createroles=None,
                user=None,
                groups=None,
                maintenance_db=None,
                login=True,
                password=None,
                valid_until=None,
                createdb=None,
            )

    def test_present__no_update(self):
        # test=True
        with patch.dict(
            postgres_user.__salt__,
            {
                "postgres.role_get": Mock(
                    return_value={
                        "can create databases": False,
                        "can create roles": False,
                        "can login": False,
                        "can update system catalogs": False,
                        "connections": None,
                        "defaults variables": {},
                        "expiry time": None,
                        "inherits privileges": True,
                        "replication": False,
                        "superuser": False,
                    }
                ),
                "postgres.user_update": MagicMock(),
            },
        ):
            with patch.dict(postgres_user.__opts__, {"test": True}):
                ret = postgres_user.present("foo", login=False, replication=False)
                self.assertEqual(
                    ret,
                    {
                        "comment": "User foo is already present",
                        "changes": {},
                        "name": "foo",
                        "result": True,
                    },
                )
                self.assertEqual(self.salt_stub["postgres.user_update"].call_count, 0)

            # test=False
            ret = postgres_user.present("foo", login=False, replication=False)
            self.assertEqual(
                ret,
                {
                    "comment": "User foo is already present",
                    "changes": {},
                    "name": "foo",
                    "result": True,
                },
            )
            self.assertEqual(self.salt_stub["postgres.user_update"].call_count, 0)


class PostgresGroupTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        patcher = patch("salt.utils.path.which", Mock(return_value="/usr/bin/pgsql"))
        patcher.start()
        self.addCleanup(patcher.stop)
        self.salt_stub = {
            "config.option": Mock(),
            "cmd.run_all": Mock(),
            "file.chown": Mock(),
            "file.remove": Mock(),
        }
        self.addCleanup(delattr, self, "salt_stub")
        return {
            postgres_database: {},
            postgres_user: {},
            postgres_extension: {},
            postgres_schema: {},
            postgres_group: {
                "__grains__": {"os_family": "Linux"},
                "__salt__": self.salt_stub,
                "__opts__": {"test": False},
            },
        }

    def test_present__creation(self):
        # test=True
        with patch.dict(
            postgres_group.__salt__,
            {
                "postgres.role_get": Mock(return_value=None),
                "postgres.group_create": MagicMock(),
            },
        ):
            with patch.dict(postgres_group.__opts__, {"test": True}):
                ret = postgres_group.present("foo")
                self.assertEqual(
                    ret,
                    {
                        "comment": "Group foo is set to be created",
                        "changes": {},
                        "name": "foo",
                        "result": None,
                    },
                )
                self.assertEqual(self.salt_stub["postgres.group_create"].call_count, 0)

            # test=False
            ret = postgres_group.present("foo")
            self.assertEqual(
                ret,
                {
                    "comment": "The group foo has been created",
                    "changes": {},
                    "name": "foo",
                    "result": True,
                },
            )
            self.salt_stub["postgres.group_create"].assert_called_once_with(
                superuser=None,
                replication=None,
                encrypted=True,
                runas=None,
                inherit=None,
                rolepassword=None,
                port=None,
                groupname="foo",
                host=None,
                createroles=None,
                user=None,
                groups=None,
                maintenance_db=None,
                login=None,
                password=None,
                createdb=None,
            )

    def test_present__update(self):
        # test=True
        with patch.dict(
            postgres_group.__salt__,
            {
                "postgres.role_get": Mock(
                    return_value={
                        "can create databases": False,
                        "can create roles": False,
                        "can login": False,
                        "can update system catalogs": False,
                        "connections": None,
                        "defaults variables": {},
                        "expiry time": None,
                        "inherits privileges": True,
                        "replication": False,
                        "superuser": False,
                    }
                ),
                "postgres.group_update": MagicMock(),
            },
        ):
            with patch.dict(postgres_group.__opts__, {"test": True}):
                ret = postgres_group.present("foo", login=True, replication=False)
                self.assertEqual(
                    ret,
                    {
                        "comment": "Group foo is set to be updated",
                        "changes": {"foo": {"login": True}},
                        "name": "foo",
                        "result": None,
                    },
                )
                self.assertEqual(self.salt_stub["postgres.group_update"].call_count, 0)

            # test=False
            ret = postgres_group.present("foo", login=True, replication=False)
            self.assertEqual(
                ret,
                {
                    "comment": "The group foo has been updated",
                    "changes": {"foo": {"login": True}},
                    "name": "foo",
                    "result": True,
                },
            )
            self.salt_stub["postgres.group_update"].assert_called_once_with(
                superuser=None,
                replication=False,
                encrypted=True,
                runas=None,
                inherit=None,
                rolepassword=None,
                port=None,
                groupname="foo",
                host=None,
                createroles=None,
                user=None,
                groups=None,
                maintenance_db=None,
                login=True,
                password=None,
                createdb=None,
            )

    def test_present__no_update(self):
        # test=True
        with patch.dict(
            postgres_group.__salt__,
            {
                "postgres.role_get": Mock(
                    return_value={
                        "can create databases": False,
                        "can create roles": False,
                        "can login": False,
                        "can update system catalogs": False,
                        "connections": None,
                        "defaults variables": {},
                        "expiry time": None,
                        "inherits privileges": True,
                        "replication": False,
                        "superuser": False,
                    }
                ),
                "postgres.group_update": MagicMock(),
            },
        ):
            with patch.dict(postgres_group.__opts__, {"test": True}):
                ret = postgres_group.present("foo", login=False, replication=False)
                self.assertEqual(
                    ret,
                    {
                        "comment": "Group foo is already present",
                        "changes": {},
                        "name": "foo",
                        "result": True,
                    },
                )
                self.assertEqual(self.salt_stub["postgres.group_update"].call_count, 0)

            # test=False
            ret = postgres_group.present("foo", login=False, replication=False)
            self.assertEqual(
                ret,
                {
                    "comment": "Group foo is already present",
                    "changes": {},
                    "name": "foo",
                    "result": True,
                },
            )
            self.assertEqual(self.salt_stub["postgres.group_update"].call_count, 0)


class PostgresExtensionTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        patcher = patch("salt.utils.path.which", Mock(return_value="/usr/bin/pgsql"))
        patcher.start()
        self.addCleanup(patcher.stop)
        self.salt_stub = {
            "config.option": Mock(),
            "cmd.run_all": Mock(),
            "file.chown": Mock(),
            "file.remove": Mock(),
        }
        self.addCleanup(delattr, self, "salt_stub")
        return {
            postgres_database: {},
            postgres_user: {},
            postgres_group: {},
            postgres_schema: {},
            postgres_extension: {
                "__grains__": {"os_family": "Linux"},
                "__salt__": self.salt_stub,
                "__opts__": {"test": False},
            },
        }

    def test_present_failed(self):
        """
        scenario of creating upgrading extensions with possible schema and
        version specifications
        """
        with patch.dict(
            postgres_extension.__salt__,
            {
                "postgres.create_metadata": Mock(
                    side_effect=[
                        [postgresmod._EXTENSION_NOT_INSTALLED],
                        [
                            postgresmod._EXTENSION_TO_MOVE,
                            postgresmod._EXTENSION_INSTALLED,
                        ],
                    ]
                ),
                "postgres.create_extension": Mock(side_effect=[False, False]),
            },
        ):
            ret = postgres_extension.present("foo")
            self.assertEqual(
                ret,
                {
                    "comment": "Failed to install extension foo",
                    "changes": {},
                    "name": "foo",
                    "result": False,
                },
            )
            ret = postgres_extension.present("foo")
            self.assertEqual(
                ret,
                {
                    "comment": "Failed to upgrade extension foo",
                    "changes": {},
                    "name": "foo",
                    "result": False,
                },
            )

    def test_present(self):
        """
        scenario of creating upgrading extensions with possible schema and
        version specifications
        """
        with patch.dict(
            postgres_extension.__salt__,
            {
                "postgres.create_metadata": Mock(
                    side_effect=[
                        [postgresmod._EXTENSION_NOT_INSTALLED],
                        [postgresmod._EXTENSION_INSTALLED],
                        [
                            postgresmod._EXTENSION_TO_MOVE,
                            postgresmod._EXTENSION_INSTALLED,
                        ],
                    ]
                ),
                "postgres.create_extension": Mock(side_effect=[True, True, True]),
            },
        ):
            ret = postgres_extension.present("foo")
            self.assertEqual(
                ret,
                {
                    "comment": "The extension foo has been installed",
                    "changes": {"foo": "Installed"},
                    "name": "foo",
                    "result": True,
                },
            )
            ret = postgres_extension.present("foo")
            self.assertEqual(
                ret,
                {
                    "comment": "Extension foo is already present",
                    "changes": {},
                    "name": "foo",
                    "result": True,
                },
            )
            ret = postgres_extension.present("foo")
            self.assertEqual(
                ret,
                {
                    "comment": "The extension foo has been upgraded",
                    "changes": {"foo": "Upgraded"},
                    "name": "foo",
                    "result": True,
                },
            )

    def test_presenttest(self):
        """
        scenario of creating upgrading extensions with possible schema and
        version specifications
        """
        with patch.dict(
            postgres_extension.__salt__,
            {
                "postgres.create_metadata": Mock(
                    side_effect=[
                        [postgresmod._EXTENSION_NOT_INSTALLED],
                        [postgresmod._EXTENSION_INSTALLED],
                        [
                            postgresmod._EXTENSION_TO_MOVE,
                            postgresmod._EXTENSION_INSTALLED,
                        ],
                    ]
                ),
                "postgres.create_extension": Mock(side_effect=[True, True, True]),
            },
        ):
            with patch.dict(postgres_extension.__opts__, {"test": True}):
                ret = postgres_extension.present("foo")
                self.assertEqual(
                    ret,
                    {
                        "comment": "Extension foo is set to be installed",
                        "changes": {},
                        "name": "foo",
                        "result": None,
                    },
                )
                ret = postgres_extension.present("foo")
                self.assertEqual(
                    ret,
                    {
                        "comment": "Extension foo is already present",
                        "changes": {},
                        "name": "foo",
                        "result": True,
                    },
                )
                ret = postgres_extension.present("foo")
                self.assertEqual(
                    ret,
                    {
                        "comment": "Extension foo is set to be upgraded",
                        "changes": {},
                        "name": "foo",
                        "result": None,
                    },
                )

    def test_absent(self):
        """
        scenario of creating upgrading extensions with possible schema and
        version specifications
        """
        with patch.dict(
            postgres_extension.__salt__,
            {
                "postgres.is_installed_extension": Mock(side_effect=[True, False]),
                "postgres.drop_extension": Mock(side_effect=[True, True]),
            },
        ):
            ret = postgres_extension.absent("foo")
            self.assertEqual(
                ret,
                {
                    "comment": "Extension foo has been removed",
                    "changes": {"foo": "Absent"},
                    "name": "foo",
                    "result": True,
                },
            )
            ret = postgres_extension.absent("foo")
            self.assertEqual(
                ret,
                {
                    "comment": (
                        "Extension foo is not present, " "so it cannot be removed"
                    ),
                    "changes": {},
                    "name": "foo",
                    "result": True,
                },
            )

    def test_absent_failed(self):
        """
        scenario of creating upgrading extensions with possible schema and
        version specifications
        """
        with patch.dict(postgres_extension.__opts__, {"test": False}):
            with patch.dict(
                postgres_extension.__salt__,
                {
                    "postgres.is_installed_extension": Mock(side_effect=[True, True]),
                    "postgres.drop_extension": Mock(side_effect=[False, False]),
                },
            ):
                ret = postgres_extension.absent("foo")
                self.assertEqual(
                    ret,
                    {
                        "comment": "Extension foo failed to be removed",
                        "changes": {},
                        "name": "foo",
                        "result": False,
                    },
                )

    def test_absent_failedtest(self):
        with patch.dict(
            postgres_extension.__salt__,
            {
                "postgres.is_installed_extension": Mock(side_effect=[True, True]),
                "postgres.drop_extension": Mock(side_effect=[False, False]),
            },
        ):
            with patch.dict(postgres_extension.__opts__, {"test": True}):
                ret = postgres_extension.absent("foo")
            self.assertEqual(
                ret,
                {
                    "comment": "Extension foo is set to be removed",
                    "changes": {},
                    "name": "foo",
                    "result": None,
                },
            )


class PostgresSchemaTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        patcher = patch("salt.utils.path.which", Mock(return_value="/usr/bin/pgsql"))
        patcher.start()
        self.addCleanup(patcher.stop)
        self.salt_stub = {
            "config.option": Mock(),
            "cmd.run_all": Mock(),
            "file.chown": Mock(),
            "file.remove": Mock(),
        }
        self.addCleanup(delattr, self, "salt_stub")
        return {
            postgres_database: {},
            postgres_user: {},
            postgres_extension: {},
            postgres_group: {},
            postgres_schema: {
                "__grains__": {"os_family": "Linux"},
                "__salt__": self.salt_stub,
                "__opts__": {"test": False},
            },
        }

    def test_present_creation(self):
        with patch.dict(
            postgres_schema.__salt__,
            {
                "postgres.schema_get": Mock(return_value=None),
                "postgres.schema_create": MagicMock(),
            },
        ):
            ret = postgres_schema.present("dbname", "foo")
            self.assertEqual(
                ret,
                {
                    "comment": "Schema foo has been created in database dbname",
                    "changes": {"foo": "Present"},
                    "dbname": "dbname",
                    "name": "foo",
                    "result": True,
                },
            )
            self.assertEqual(self.salt_stub["postgres.schema_create"].call_count, 1)

    def test_present_nocreation(self):
        with patch.dict(
            postgres_schema.__salt__,
            {
                "postgres.schema_get": Mock(
                    return_value={"foo": {"acl": "", "owner": "postgres"}}
                ),
                "postgres.schema_create": MagicMock(),
            },
        ):
            ret = postgres_schema.present("dbname", "foo")
            self.assertEqual(
                ret,
                {
                    "comment": "Schema foo already exists in database dbname",
                    "changes": {},
                    "dbname": "dbname",
                    "name": "foo",
                    "result": True,
                },
            )
            self.assertEqual(self.salt_stub["postgres.schema_create"].call_count, 0)

    def test_absent_remove(self):
        with patch.dict(
            postgres_schema.__salt__,
            {
                "postgres.schema_exists": Mock(return_value=True),
                "postgres.schema_remove": MagicMock(),
            },
        ):
            ret = postgres_schema.absent("dbname", "foo")
            self.assertEqual(
                ret,
                {
                    "comment": "Schema foo has been removed from database dbname",
                    "changes": {"foo": "Absent"},
                    "dbname": "dbname",
                    "name": "foo",
                    "result": True,
                },
            )
            self.assertEqual(self.salt_stub["postgres.schema_remove"].call_count, 1)

    def test_absent_noremove(self):
        with patch.dict(
            postgres_schema.__salt__,
            {
                "postgres.schema_exists": Mock(return_value=False),
                "postgres.schema_remove": MagicMock(),
            },
        ):
            ret = postgres_schema.absent("dbname", "foo")
            self.assertEqual(
                ret,
                {
                    "comment": "Schema foo is not present in database dbname,"
                    " so it cannot be removed",
                    "changes": {},
                    "dbname": "dbname",
                    "name": "foo",
                    "result": True,
                },
            )
            self.assertEqual(self.salt_stub["postgres.schema_remove"].call_count, 0)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Megan Wilhite<mwilhite@saltstack.com>
"""

# Import Python libs
from __future__ import absolute_import

# Import Salt Libs
import salt.modules.mac_service as mac_service

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class MacServiceTestCase(TestCase, LoaderModuleMockMixin):
    """
    TestCase for salt.modules.mac_service module
    """

    def setup_loader_modules(self):
        return {mac_service: {}}

    def test_service_disabled_when_enabled(self):
        """
        test service.disabled when service is enabled
        """
        srv_name = "com.apple.atrun"
        cmd = 'disabled services = {\n\t"com.saltstack.salt.minion" => false\n\t"com.apple.atrun" => false\n{'

        with patch.object(mac_service, "launchctl", MagicMock(return_value=cmd)):
            self.assertFalse(mac_service.disabled(srv_name))

    def test_service_disabled_when_disabled(self):
        """
        test service.disabled when service is disabled
        """
        srv_name = "com.apple.atrun"
        cmd = 'disabled services = {\n\t"com.saltstack.salt.minion" => false\n\t"com.apple.atrun" => true\n{'

        with patch.object(mac_service, "launchctl", MagicMock(return_value=cmd)):
            self.assertTrue(mac_service.disabled(srv_name))

    def test_service_disabled_srvname_wrong(self):
        """
        test service.disabled when service is just slightly wrong
        """
        srv_names = ["com.apple.atru", "com", "apple"]
        cmd = 'disabled services = {\n\t"com.saltstack.salt.minion" => false\n\t"com.apple.atrun" => true\n}'
        for name in srv_names:
            with patch.object(mac_service, "launchctl", MagicMock(return_value=cmd)):
                self.assertFalse(mac_service.disabled(name))

    def test_service_disabled_status_upper_case(self):
        """
        test service.disabled when disabled status is uppercase
        """
        srv_name = "com.apple.atrun"
        cmd = 'disabled services = {\n\t"com.saltstack.salt.minion" => false\n\t"com.apple.atrun" => True\n{'

        with patch.object(mac_service, "launchctl", MagicMock(return_value=cmd)):
            self.assertTrue(mac_service.disabled(srv_name))

    def test_service_keep_alive_pathstate_file_rm(self):
        """
        test _always_running_service when keep_alive
        has pathstate set in plist file and file doesn't exist
        """
        srv_name = "com.apple.atrun"
        info = {
            "plist": {
                "EnableTransactions": True,
                "ProgramArguments": ["/usr/libexec/ntpd-wrapper"],
                "Label": "org.ntp.ntpd",
                "KeepAlive": {"PathState": {"/private/etc/ntp.conf": True}},
            }
        }

        with patch.object(mac_service, "show", MagicMock(return_value=info)):
            with patch("os.path.exists", MagicMock(return_value=False)):
                assert mac_service._always_running_service(srv_name) is False

    def test_service_keep_alive_empty(self):
        """
        test _always_running_service when keep_alive
        is empty
        """
        srv_name = "com.apple.atrun"
        info = {
            "plist": {
                "EnableTransactions": True,
                "ProgramArguments": ["/usr/libexec/ntpd-wrapper"],
                "Label": "org.ntp.ntpd",
                "KeepAlive": {},
            }
        }

        with patch.object(mac_service, "show", MagicMock(return_value=info)):
            with patch("os.path.exists", MagicMock(return_value=False)):
                assert mac_service._always_running_service(srv_name) is False

    def test_service_keep_alive_pathstate_false(self):
        """
        test _always_running_service when keep_alive
        has pathstate set in plist file and file is false
        """
        srv_name = "com.apple.atrun"
        info = {
            "plist": {
                "EnableTransactions": True,
                "ProgramArguments": ["/usr/libexec/ntpd-wrapper"],
                "Label": "org.ntp.ntpd",
                "KeepAlive": {"PathState": {"/private/etc/ntp.conf": False}},
            }
        }

        with patch.object(mac_service, "show", MagicMock(return_value=info)):
            with patch("os.path.exists", MagicMock(return_value=False)):
                assert mac_service._always_running_service(srv_name) is True

    def test_service_keep_alive_pathstate(self):
        """
        test _always_running_service when keep_alive
        has pathstate set in plist file
        """
        srv_name = "com.apple.atrun"
        info = {
            "plist": {
                "EnableTransactions": True,
                "ProgramArguments": ["/usr/libexec/ntpd-wrapper"],
                "Label": "org.ntp.ntpd",
                "KeepAlive": {"PathState": {"/private/etc/ntp.conf": True}},
            }
        }

        with patch.object(mac_service, "show", MagicMock(return_value=info)):
            with patch("os.path.exists", MagicMock(return_value=True)):
                assert mac_service._always_running_service(srv_name) is True

    def test_service_keep_alive(self):
        """
        test _always_running_service when keep_alive set
        """
        srv_name = "com.apple.atrun"
        info = {
            "plist": {
                "EnableTransactions": True,
                "ProgramArguments": ["/usr/libexec/ntpd-wrapper"],
                "Label": "org.ntp.ntpd",
                "KeepAlive": True,
            }
        }

        with patch.object(mac_service, "show", MagicMock(return_value=info)):
            assert mac_service._always_running_service(srv_name) is True

    def test_service_keep_alive_false(self):
        """
        test _always_running_service when keep_alive False
        """
        srv_name = "com.apple.atrun"
        info = {
            "plist": {
                "EnableTransactions": True,
                "ProgramArguments": ["/usr/libexec/ntpd-wrapper"],
                "Label": "org.ntp.ntpd",
                "KeepAlive": False,
            }
        }

        with patch.object(mac_service, "show", MagicMock(return_value=info)):
            assert mac_service._always_running_service(srv_name) is False

    def test_service_keep_alive_missing(self):
        """
        test _always_running_service when keep_alive not in dict
        """
        srv_name = "com.apple.atrun"
        info = {
            "plist": {
                "EnableTransactions": True,
                "ProgramArguments": ["/usr/libexec/ntpd-wrapper"],
                "Label": "org.ntp.ntpd",
            }
        }

        with patch.object(mac_service, "show", MagicMock(return_value=info)):
            assert mac_service._always_running_service(srv_name) is False

    def test_service_keep_alive_wrong_setting(self):
        """
        test _always_running_service when keep_alive
        has pathstate set in plist file
        """
        srv_name = "com.apple.atrun"
        info = {
            "plist": {
                "EnableTransactions": True,
                "ProgramArguments": ["/usr/libexec/ntpd-wrapper"],
                "Label": "org.ntp.ntpd",
                "KeepAlive": {"Doesnotexist": {"doesnt_exist": True}},
            }
        }

        with patch.object(mac_service, "show", MagicMock(return_value=info)):
            assert mac_service._always_running_service(srv_name) is False



# -*- coding: utf-8 -*-
"""
    tests.pytests.conftest
    ~~~~~~~~~~~~~~~~~~~~~~
"""
import pathlib

import pytest
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import TestCase

PYTESTS_SUITE_PATH = pathlib.Path(__file__).parent


@pytest.hookimpl(hookwrapper=True, trylast=True)
def pytest_collection_modifyitems(config, items):
    """
    called after collection has been performed, may filter or re-order
    the items in-place.

    :param _pytest.main.Session session: the pytest session object
    :param _pytest.config.Config config: pytest config object
    :param List[_pytest.nodes.Item] items: list of item objects
    """
    # Let PyTest or other plugins handle the initial collection
    yield

    # Check each collected item that's under this package to ensure that none is using TestCase as the base class
    for item in items:
        if not str(item.fspath).startswith(str(PYTESTS_SUITE_PATH)):
            continue
        if not item.cls:
            # The test item is not part of a class
            continue

        if issubclass(item.cls, TestCase):
            raise RuntimeError(
                "The tests under {} MUST NOT use unittest's TestCase class or a subclass of it.".format(
                    pathlib.Path(str(item.fspath)).relative_to(RUNTIME_VARS.CODE_DIR)
                )
            )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Megan Wilhite <mwilhite@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
from salt.ext import six

# Create the cloud instance name to be used throughout the tests
from tests.integration.cloud.helpers.cloud_test_base import TIMEOUT, CloudTest


class VMWareTest(CloudTest):
    """
    Integration tests for the vmware cloud provider in Salt-Cloud
    """

    PROVIDER = "vmware"
    REQUIRED_PROVIDER_CONFIG_ITEMS = ("password", "user", "url")

    def test_instance(self):
        """
        Tests creating and deleting an instance on vmware and installing salt
        """
        # create the instance
        disk_datastore = self.config["vmware-test"]["devices"]["disk"]["Hard disk 2"][
            "datastore"
        ]

        ret_val = self.run_cloud(
            "-p vmware-test {0}".format(self.instance_name), timeout=TIMEOUT
        )
        disk_datastore_str = "                [{0}] {1}/Hard disk 2-flat.vmdk".format(
            disk_datastore, self.instance_name
        )

        # check if instance returned with salt installed
        self.assertInstanceExists(ret_val)
        self.assertIn(
            disk_datastore_str,
            ret_val,
            msg="Hard Disk 2 did not use the Datastore {0} ".format(disk_datastore),
        )

        self.assertDestroyInstance()

    def test_snapshot(self):
        """
        Tests creating snapshot and creating vm with --no-deploy
        """
        # create the instance
        ret_val = self.run_cloud(
            "-p vmware-test {0} --no-deploy".format(self.instance_name), timeout=TIMEOUT
        )

        # check if instance returned with salt installed
        self.assertInstanceExists(ret_val)

        create_snapshot = self.run_cloud(
            "-a create_snapshot {0} \
                                         snapshot_name='Test Cloud' \
                                         memdump=True -y".format(
                self.instance_name
            ),
            timeout=TIMEOUT,
        )
        s_ret_str = "Snapshot created successfully"

        self.assertIn(s_ret_str, six.text_type(create_snapshot))

        self.assertDestroyInstance()



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import re

# Import Salt libs
import salt.utils.platform

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.unit import skipIf


@skipIf(not salt.utils.platform.is_windows(), "windows test only")
class WinIPTest(ModuleCase):
    """
    Tests for salt.modules.win_ip
    """

    def test_get_default_gateway(self):
        """
        Test getting default gateway
        """
        ip = re.compile(r"^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$")
        ret = self.run_function("ip.get_default_gateway")
        assert ip.match(ret)

    def test_ip_is_enabled(self):
        """
        Test ip.is_enabled
        """
        assert self.run_function("ip.is_enabled", ["Ethernet"])
        assert "not found" in self.run_function("ip.is_enabled", ["doesnotexist"])



# -*- coding: utf-8 -*-
"""
Tests for the salt-run command
"""
from __future__ import absolute_import, print_function, unicode_literals

import logging

import pytest
from tests.support.case import ShellCase
from tests.support.helpers import slowTest

log = logging.getLogger(__name__)


@pytest.mark.usefixtures("salt_sub_minion")
class CacheTest(ShellCase):
    """
    Test the cache runner.
    """

    @slowTest
    def test_cache(self):
        """
        Store, list, fetch, then flush data
        """
        # Store the data
        ret = self.run_run_plus(
            "cache.store",
            bank="cachetest/runner",
            key="test_cache",
            data="The time has come the walrus said",
        )
        # Make sure we can see the new key
        ret = self.run_run_plus("cache.list", bank="cachetest/runner")
        self.assertIn("test_cache", ret["return"])
        # Make sure we can see the new data
        ret = self.run_run_plus(
            "cache.fetch", bank="cachetest/runner", key="test_cache"
        )
        self.assertIn("The time has come the walrus said", ret["return"])
        # Make sure we can delete the data
        ret = self.run_run_plus(
            "cache.flush", bank="cachetest/runner", key="test_cache"
        )
        ret = self.run_run_plus("cache.list", bank="cachetest/runner")
        self.assertNotIn("test_cache", ret["return"])

    @slowTest
    def test_cache_invalid(self):
        """
        Store, list, fetch, then flush data
        """
        # Store the data
        ret = self.run_run_plus("cache.store",)
        # Make sure we can see the new key
        expected = "Passed invalid arguments:"
        self.assertIn(expected, ret["return"])

    @slowTest
    def test_grains(self):
        """
        Test cache.grains
        """
        # Store the data
        ret = self.run_run_plus("cache.grains", tgt="minion")

        self.assertIn("minion", ret["return"])

    @slowTest
    def test_pillar(self):
        """
        Test cache.pillar
        """
        # Store the data
        ret = self.run_run_plus("cache.pillar", tgt="minion")

        assert "minion" in ret["return"]
        assert "sub_minion" not in ret["return"]

    @slowTest
    def test_pillar_no_tgt(self):
        """
        Test cache.pillar when no tgt is
        supplied. This should return pillar
        data for all minions
        """
        # Store the data
        ret = self.run_run_plus("cache.pillar",)

        assert all(x in ret["return"] for x in ["minion", "sub_minion"])

    @slowTest
    def test_pillar_minion_noexist(self):
        """
        Test cache.pillar when the target does not exist
        """
        ret = self.run_run_plus("cache.pillar", tgt="doesnotexist")

        assert "minion" not in ret["return"]
        assert "sub_minion" not in ret["return"]

    @slowTest
    def test_pillar_minion_tgt_type_pillar(self):
        """
        Test cache.pillar when the target exists
        and tgt_type is pillar
        """
        ret = self.run_run_plus("cache.pillar", tgt="monty:python", tgt_type="pillar",)

        assert all(x in ret["return"] for x in ["minion", "sub_minion"])

    @slowTest
    def test_mine(self):
        """
        Test cache.mine
        """
        # Store the data
        ret = self.run_run_plus("cache.mine", tgt="minion")

        self.assertIn("minion", ret["return"])



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.modjk as modjk
from salt.ext import six

# Import Salt Testing Libs
from tests.support.unit import TestCase

if six.PY2:
    LIST_NOT_STR = "workers should be a list not a <type 'unicode'>"
else:
    LIST_NOT_STR = "workers should be a list not a <class 'str'>"


class ModjkTestCase(TestCase):
    """
    Test cases for salt.states.modjk
    """

    # 'worker_stopped' function tests: 1

    def test_worker_stopped(self):
        """
        Test to stop all the workers in the modjk load balancer
        """
        name = "loadbalancer"

        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        ret.update({"comment": LIST_NOT_STR})
        self.assertDictEqual(modjk.worker_stopped(name, "app1"), ret)

    # 'worker_activated' function tests: 1

    def test_worker_activated(self):
        """
        Test to activate all the workers in the modjk load balancer
        """
        name = "loadbalancer"

        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        ret.update({"comment": LIST_NOT_STR})
        self.assertDictEqual(modjk.worker_activated(name, "app1"), ret)

    # 'worker_disabled' function tests: 1

    def test_worker_disabled(self):
        """
        Test to disable all the workers in the modjk load balancer
        """
        name = "loadbalancer"

        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        ret.update({"comment": LIST_NOT_STR})
        self.assertDictEqual(modjk.worker_disabled(name, "app1"), ret)

    # 'worker_recover' function tests: 1

    def test_worker_recover(self):
        """
        Test to recover all the workers in the modjk load balancer
        """
        name = "loadbalancer"

        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        ret.update({"comment": LIST_NOT_STR})
        self.assertDictEqual(modjk.worker_recover(name, "app1"), ret)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.rabbitmq_vhost as rabbitmq_vhost

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class RabbitmqVhostTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.rabbitmq_vhost
    """

    def setup_loader_modules(self):
        return {rabbitmq_vhost: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to ensure the RabbitMQ VHost exists.
        """
        name = "virtual_host"

        ret = {
            "name": name,
            "changes": {"new": "virtual_host", "old": ""},
            "result": None,
            "comment": "Virtual Host 'virtual_host' will be created.",
        }

        mock = MagicMock(return_value=False)
        with patch.dict(rabbitmq_vhost.__salt__, {"rabbitmq.vhost_exists": mock}):
            with patch.dict(rabbitmq_vhost.__opts__, {"test": True}):
                self.assertDictEqual(rabbitmq_vhost.present(name), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to ensure the named user is absent.
        """
        name = "myqueue"

        ret = {
            "name": name,
            "changes": {},
            "result": True,
            "comment": "Virtual Host '{0}' is not present.".format(name),
        }

        mock = MagicMock(return_value=False)
        with patch.dict(rabbitmq_vhost.__salt__, {"rabbitmq.vhost_exists": mock}):
            self.assertDictEqual(rabbitmq_vhost.absent(name), ret)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.lvs_service as lvs_service

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class LvsServiceTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.lvs_service
    """

    def setup_loader_modules(self):
        return {lvs_service: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to ensure that the named service is present.
        """
        name = "lvsrs"

        ret = {"name": name, "result": True, "comment": "", "changes": {}}

        mock_check = MagicMock(
            side_effect=[
                True,
                True,
                True,
                False,
                True,
                False,
                True,
                False,
                False,
                False,
                False,
            ]
        )
        mock_edit = MagicMock(side_effect=[True, False])
        mock_add = MagicMock(side_effect=[True, False])
        with patch.dict(
            lvs_service.__salt__,
            {
                "lvs.check_service": mock_check,
                "lvs.edit_service": mock_edit,
                "lvs.add_service": mock_add,
            },
        ):
            with patch.dict(lvs_service.__opts__, {"test": True}):
                comt = "LVS Service lvsrs is present"
                ret.update({"comment": comt})
                self.assertDictEqual(lvs_service.present(name), ret)

                comt = "LVS Service lvsrs is present but some " "options should update"
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(lvs_service.present(name), ret)

            with patch.dict(lvs_service.__opts__, {"test": False}):
                comt = "LVS Service lvsrs has been updated"
                ret.update(
                    {"comment": comt, "result": True, "changes": {"lvsrs": "Update"}}
                )
                self.assertDictEqual(lvs_service.present(name), ret)

                comt = "LVS Service lvsrs update failed"
                ret.update({"comment": comt, "result": False, "changes": {}})
                self.assertDictEqual(lvs_service.present(name), ret)

            with patch.dict(lvs_service.__opts__, {"test": True}):
                comt = "LVS Service lvsrs is not present and needs" " to be created"
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(lvs_service.present(name), ret)

            with patch.dict(lvs_service.__opts__, {"test": False}):
                comt = "LVS Service lvsrs has been created"
                ret.update(
                    {"comment": comt, "result": True, "changes": {"lvsrs": "Present"}}
                )
                self.assertDictEqual(lvs_service.present(name), ret)

                comt = "LVS Service lvsrs create failed(False)"
                ret.update({"comment": comt, "result": False, "changes": {}})
                self.assertDictEqual(lvs_service.present(name), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to ensure the LVS Real Server in specified service is absent.
        """
        name = "lvsrs"

        ret = {"name": name, "result": None, "comment": "", "changes": {}}

        mock_check = MagicMock(side_effect=[True, True, True, False])
        mock_delete = MagicMock(side_effect=[True, False])
        with patch.dict(
            lvs_service.__salt__,
            {"lvs.check_service": mock_check, "lvs.delete_service": mock_delete},
        ):
            with patch.dict(lvs_service.__opts__, {"test": True}):
                comt = "LVS Service lvsrs is present and needs to be removed"
                ret.update({"comment": comt})
                self.assertDictEqual(lvs_service.absent(name), ret)

            with patch.dict(lvs_service.__opts__, {"test": False}):
                comt = "LVS Service lvsrs has been removed"
                ret.update(
                    {"comment": comt, "result": True, "changes": {"lvsrs": "Absent"}}
                )
                self.assertDictEqual(lvs_service.absent(name), ret)

                comt = "LVS Service lvsrs removed failed(False)"
                ret.update({"comment": comt, "result": False, "changes": {}})
                self.assertDictEqual(lvs_service.absent(name), ret)

            comt = "LVS Service lvsrs is not present, so it cannot be removed"
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(lvs_service.absent(name), ret)



# -*- coding: utf-8 -*-
"""
Base classes for gitfs/git_pillar integration tests
"""

from __future__ import absolute_import, print_function, unicode_literals

import copy
import errno
import logging
import os
import pprint
import shutil
import sys
import tempfile
import textwrap

import salt.ext.six as six
import salt.utils.files
import salt.utils.path
import salt.utils.yaml
from salt.fileserver import gitfs
from salt.pillar import git_pillar
from saltfactories.utils.ports import get_unused_localhost_port
from saltfactories.utils.processes.bases import FactoryDaemonScriptBase
from saltfactories.utils.processes.helpers import start_daemon, terminate_process
from saltfactories.utils.processes.sshd import SshdDaemon
from tests.support.case import ModuleCase
from tests.support.helpers import patched_environ, requires_system_grains
from tests.support.mixins import (
    AdaptedConfigurationTestCaseMixin,
    LoaderModuleMockMixin,
    SaltReturnAssertsMixin,
)
from tests.support.mock import patch
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import SkipTest

try:
    import psutil
except ImportError:
    pass

log = logging.getLogger(__name__)

USERNAME = "gitpillaruser"
PASSWORD = "saltrules"

_OPTS = {
    "__role": "minion",
    "environment": None,
    "pillarenv": None,
    "hash_type": "sha256",
    "file_roots": {},
    "state_top": "top.sls",
    "state_top_saltenv": None,
    "renderer": "yaml_jinja",
    "renderer_whitelist": [],
    "renderer_blacklist": [],
    "pillar_merge_lists": False,
    "git_pillar_base": "master",
    "git_pillar_branch": "master",
    "git_pillar_env": "",
    "git_pillar_fallback": "",
    "git_pillar_root": "",
    "git_pillar_ssl_verify": True,
    "git_pillar_global_lock": True,
    "git_pillar_user": "",
    "git_pillar_password": "",
    "git_pillar_insecure_auth": False,
    "git_pillar_privkey": "",
    "git_pillar_pubkey": "",
    "git_pillar_passphrase": "",
    "git_pillar_refspecs": [
        "+refs/heads/*:refs/remotes/origin/*",
        "+refs/tags/*:refs/tags/*",
    ],
    "git_pillar_includes": True,
}
PROC_TIMEOUT = 10


class UwsgiDaemon(FactoryDaemonScriptBase):
    def __init__(self, *args, **kwargs):
        config_dir = kwargs.pop("config_dir")
        check_port = kwargs.pop("check_port")
        super(UwsgiDaemon, self).__init__(*args, **kwargs)
        self.config_dir = config_dir
        self.check_port = check_port

    def get_log_prefix(self):
        return "[uWSGI] "

    def get_base_script_args(self):
        """
        Returns any additional arguments to pass to the CLI script
        """
        return ["--yaml", os.path.join(self.config_dir, "uwsgi.yml")]

    def get_check_ports(self):
        """
        Return a list of ports to check against to ensure the daemon is running
        """
        return [self.check_port]


class NginxDaemon(FactoryDaemonScriptBase):
    def __init__(self, *args, **kwargs):
        config_dir = kwargs.pop("config_dir")
        check_port = kwargs.pop("check_port")
        super(NginxDaemon, self).__init__(*args, **kwargs)
        self.config_dir = config_dir
        self.check_port = check_port

    def get_log_prefix(self):
        return "[Nginx] "

    def get_base_script_args(self):
        """
        Returns any additional arguments to pass to the CLI script
        """
        return ["-c", os.path.join(self.config_dir, "nginx.conf")]

    def get_check_ports(self):
        """
        Return a list of ports to check against to ensure the daemon is running
        """
        return [self.check_port]


class SaltClientMixin(ModuleCase):

    client = None

    @classmethod
    @requires_system_grains
    def setUpClass(cls, grains=None):  # pylint: disable=arguments-differ
        # Cent OS 6 has too old a version of git to handle the make_repo code, as
        # it lacks the -c option for git itself.
        make_repo = getattr(cls, "make_repo", None)
        if (
            callable(make_repo)
            and grains["os_family"] == "RedHat"
            and grains["osmajorrelease"] < 7
        ):
            raise SkipTest("RHEL < 7 has too old a version of git to run these tests")
        # Late import
        import salt.client

        mopts = AdaptedConfigurationTestCaseMixin.get_config(
            "master", from_scratch=True
        )
        cls.user = mopts["user"]
        cls.client = salt.client.get_local_client(mopts=mopts)

    @classmethod
    def tearDownClass(cls):
        cls.client = None

    @classmethod
    def cls_run_function(cls, function, *args, **kwargs):
        orig = cls.client.cmd("minion", function, arg=args, timeout=300, kwarg=kwargs)
        return orig["minion"]


class SSHDMixin(SaltClientMixin, SaltReturnAssertsMixin):
    """
    Functions to stand up an SSHD server to serve up git repos for tests.
    """

    sshd_proc = None
    prep_states_ran = False
    known_hosts_setup = False

    @classmethod
    def setUpClass(cls):  # pylint: disable=arguments-differ
        super(SSHDMixin, cls).setUpClass()
        try:
            log.info("%s: prep_server()", cls.__name__)
            cls.sshd_bin = salt.utils.path.which("sshd")
            cls.sshd_config_dir = tempfile.mkdtemp(dir=RUNTIME_VARS.TMP)
            cls.sshd_config = os.path.join(cls.sshd_config_dir, "sshd_config")
            cls.sshd_port = get_unused_localhost_port(cached_seconds=120)
            cls.url = "ssh://{username}@127.0.0.1:{port}/~/repo.git".format(
                username=cls.username, port=cls.sshd_port
            )
            cls.url_extra_repo = "ssh://{username}@127.0.0.1:{port}/~/extra_repo.git".format(
                username=cls.username, port=cls.sshd_port
            )
            home = "/root/.ssh"
            cls.ext_opts = {
                "url": cls.url,
                "url_extra_repo": cls.url_extra_repo,
                "privkey_nopass": os.path.join(home, cls.id_rsa_nopass),
                "pubkey_nopass": os.path.join(home, cls.id_rsa_nopass + ".pub"),
                "privkey_withpass": os.path.join(home, cls.id_rsa_withpass),
                "pubkey_withpass": os.path.join(home, cls.id_rsa_withpass + ".pub"),
                "passphrase": cls.passphrase,
            }

            if cls.prep_states_ran is False:
                ret = cls.cls_run_function(
                    "state.apply",
                    mods="git_pillar.ssh",
                    pillar={
                        "git_pillar": {
                            "git_ssh": cls.git_ssh,
                            "id_rsa_nopass": cls.id_rsa_nopass,
                            "id_rsa_withpass": cls.id_rsa_withpass,
                            "sshd_bin": cls.sshd_bin,
                            "sshd_port": cls.sshd_port,
                            "sshd_config_dir": cls.sshd_config_dir,
                            "master_user": cls.user,
                            "user": cls.username,
                        }
                    },
                )
                assert next(six.itervalues(ret))["result"] is True
                cls.prep_states_ran = True
                log.info("%s: States applied", cls.__name__)
            if cls.sshd_proc is not None:
                if not psutil.pid_exists(cls.sshd_proc.pid):
                    log.info(
                        "%s: sshd started but appears to be dead now. Will try to restart it.",
                        cls.__name__,
                    )
                    cls.sshd_proc = None
            if cls.sshd_proc is None:
                cls.sshd_proc = start_daemon(
                    cls.sshd_bin,
                    SshdDaemon,
                    config_dir=cls.sshd_config_dir,
                    serve_port=cls.sshd_port,
                )
                log.info("%s: sshd started", cls.__name__)
        except AssertionError:
            cls.tearDownClass()
            six.reraise(*sys.exc_info())

        if cls.known_hosts_setup is False:
            known_hosts_ret = cls.cls_run_function(
                "ssh.set_known_host",
                user=cls.user,
                hostname="127.0.0.1",
                port=cls.sshd_port,
                enc="ssh-rsa",
                fingerprint="fd:6f:7f:5d:06:6b:f2:06:0d:26:93:9e:5a:b5:19:46",
                hash_known_hosts=False,
                fingerprint_hash_type="md5",
            )
            if "error" in known_hosts_ret:
                cls.tearDownClass()
                raise AssertionError(
                    "Failed to add key to {0} user's known_hosts "
                    "file: {1}".format(
                        cls.master_opts["user"], known_hosts_ret["error"]
                    )
                )
            cls.known_hosts_setup = True

    @classmethod
    def tearDownClass(cls):
        if cls.sshd_proc is not None:
            log.info(
                "[%s] Stopping %s",
                cls.sshd_proc.get_log_prefix(),
                cls.sshd_proc.__class__.__name__,
            )
            terminate_process(cls.sshd_proc.pid, kill_children=True, slow_stop=True)
            log.info(
                "[%s] %s stopped",
                cls.sshd_proc.get_log_prefix(),
                cls.sshd_proc.__class__.__name__,
            )
            cls.sshd_proc = None
        if cls.prep_states_ran:
            ret = cls.cls_run_function(
                "state.single", "user.absent", name=cls.username, purge=True
            )
            try:
                if ret and "minion" in ret:
                    ret_data = next(six.itervalues(ret["minion"]))
                    if not ret_data["result"]:
                        log.warning("Failed to delete test account %s", cls.username)
            except KeyError:
                log.warning(
                    "Failed to delete test account. Salt return:\n%s",
                    pprint.pformat(ret),
                )
            cls.prep_states_ran = False
            cls.known_hosts_setup = False
        shutil.rmtree(cls.sshd_config_dir, ignore_errors=True)
        ssh_dir = os.path.expanduser("~/.ssh")
        for filename in (
            cls.id_rsa_nopass,
            cls.id_rsa_nopass + ".pub",
            cls.id_rsa_withpass,
            cls.id_rsa_withpass + ".pub",
            cls.git_ssh,
        ):
            try:
                os.remove(os.path.join(ssh_dir, filename))
            except OSError as exc:
                if exc.errno != errno.ENOENT:
                    raise
        super(SSHDMixin, cls).tearDownClass()


class WebserverMixin(SaltClientMixin, SaltReturnAssertsMixin):
    """
    Functions to stand up an nginx + uWSGI + git-http-backend webserver to
    serve up git repos for tests.
    """

    nginx_proc = uwsgi_proc = None
    prep_states_ran = False

    @classmethod
    def setUpClass(cls):  # pylint: disable=arguments-differ
        """
        Set up all the webserver paths. Designed to be run once in a
        setUpClass function.
        """
        super(WebserverMixin, cls).setUpClass()
        cls.root_dir = tempfile.mkdtemp(dir=RUNTIME_VARS.TMP)
        cls.config_dir = os.path.join(cls.root_dir, "config")
        cls.nginx_conf = os.path.join(cls.config_dir, "nginx.conf")
        cls.uwsgi_conf = os.path.join(cls.config_dir, "uwsgi.yml")
        cls.git_dir = os.path.join(cls.root_dir, "git")
        cls.repo_dir = os.path.join(cls.git_dir, "repos")
        cls.venv_dir = os.path.join(cls.root_dir, "venv")
        cls.uwsgi_bin = os.path.join(cls.venv_dir, "bin", "uwsgi")
        cls.nginx_port = cls.uwsgi_port = get_unused_localhost_port(cached_seconds=120)
        cls.uwsgi_port = get_unused_localhost_port(cached_seconds=120)
        cls.url = "http://127.0.0.1:{port}/repo.git".format(port=cls.nginx_port)
        cls.url_extra_repo = "http://127.0.0.1:{port}/extra_repo.git".format(
            port=cls.nginx_port
        )
        cls.ext_opts = {"url": cls.url, "url_extra_repo": cls.url_extra_repo}
        # Add auth params if present (if so this will trigger the spawned
        # server to turn on HTTP basic auth).
        for credential_param in ("user", "password"):
            if hasattr(cls, credential_param):
                cls.ext_opts[credential_param] = getattr(cls, credential_param)
        auth_enabled = hasattr(cls, "username") and hasattr(cls, "password")
        pillar = {
            "git_pillar": {
                "config_dir": cls.config_dir,
                "git_dir": cls.git_dir,
                "venv_dir": cls.venv_dir,
                "root_dir": cls.root_dir,
                "nginx_port": cls.nginx_port,
                "uwsgi_port": cls.uwsgi_port,
                "auth_enabled": auth_enabled,
            }
        }

        # Different libexec dir for git backend on Debian-based systems
        git_core = "/usr/libexec/git-core"
        if not os.path.exists(git_core):
            git_core = "/usr/lib/git-core"

        if not os.path.exists(git_core):
            cls.tearDownClass()
            raise AssertionError(
                "{} not found. Either git is not installed, or the test "
                "class needs to be updated.".format(git_core)
            )

        pillar["git_pillar"]["git-http-backend"] = os.path.join(
            git_core, "git-http-backend"
        )
        try:
            if cls.prep_states_ran is False:
                ret = cls.cls_run_function(
                    "state.apply", mods="git_pillar.http", pillar=pillar
                )
                assert next(six.itervalues(ret))["result"] is True
                cls.prep_states_ran = True
                log.info("%s: States applied", cls.__name__)
            if cls.uwsgi_proc is not None:
                if not psutil.pid_exists(cls.uwsgi_proc.pid):
                    log.warning(
                        "%s: uWsgi started but appears to be dead now. Will try to restart it.",
                        cls.__name__,
                    )
                    cls.uwsgi_proc = None
            if cls.uwsgi_proc is None:
                cls.uwsgi_proc = start_daemon(
                    cls.uwsgi_bin,
                    UwsgiDaemon,
                    config_dir=cls.config_dir,
                    check_port=cls.uwsgi_port,
                )
                log.info("%s: %s started", cls.__name__, cls.uwsgi_bin)
            if cls.nginx_proc is not None:
                if not psutil.pid_exists(cls.nginx_proc.pid):
                    log.warning(
                        "%s: nginx started but appears to be dead now. Will try to restart it.",
                        cls.__name__,
                    )
                    cls.nginx_proc = None
            if cls.nginx_proc is None:
                cls.nginx_proc = start_daemon(
                    "nginx",
                    NginxDaemon,
                    config_dir=cls.config_dir,
                    check_port=cls.nginx_port,
                )
                log.info("%s: nginx started", cls.__name__)
        except AssertionError:
            cls.tearDownClass()
            six.reraise(*sys.exc_info())

    @classmethod
    def tearDownClass(cls):
        if cls.nginx_proc is not None:
            log.info(
                "[%s] Stopping %s",
                cls.nginx_proc.get_log_prefix(),
                cls.nginx_proc.__class__.__name__,
            )
            terminate_process(cls.nginx_proc.pid, kill_children=True, slow_stop=True)
            log.info(
                "[%s] %s stopped",
                cls.nginx_proc.get_log_prefix(),
                cls.nginx_proc.__class__.__name__,
            )
            cls.nginx_proc = None
        if cls.uwsgi_proc is not None:
            log.info(
                "[%s] Stopping %s",
                cls.uwsgi_proc.get_log_prefix(),
                cls.uwsgi_proc.__class__.__name__,
            )
            terminate_process(cls.uwsgi_proc.pid, kill_children=True, slow_stop=True)
            log.info(
                "[%s] %s stopped",
                cls.uwsgi_proc.get_log_prefix(),
                cls.uwsgi_proc.__class__.__name__,
            )
            cls.uwsgi_proc = None
        shutil.rmtree(cls.root_dir, ignore_errors=True)
        cls.prep_states_ran = False
        super(WebserverMixin, cls).tearDownClass()


class GitTestBase(ModuleCase):
    """
    Base class for all gitfs/git_pillar tests. Must be subclassed and paired
    with either SSHDMixin or WebserverMixin to provide the server.
    """

    maxDiff = None
    git_opts = '-c user.name="Foo Bar" -c user.email=foo@bar.com'
    ext_opts = {}

    def make_repo(self, root_dir, user="root"):
        raise NotImplementedError()


class GitFSTestBase(GitTestBase, LoaderModuleMockMixin):
    """
    Base class for all gitfs tests
    """

    @requires_system_grains
    def setup_loader_modules(self, grains):  # pylint: disable=W0221
        return {gitfs: {"__opts__": copy.copy(_OPTS), "__grains__": grains}}

    def make_repo(self, root_dir, user="root"):
        raise NotImplementedError()


class GitPillarTestBase(GitTestBase, LoaderModuleMockMixin):
    """
    Base class for all git_pillar tests
    """

    bare_repo = bare_repo_backup = bare_extra_repo = bare_extra_repo_backup = None
    admin_repo = admin_repo_backup = admin_extra_repo = admin_extra_repo_backup = None

    @requires_system_grains
    def setup_loader_modules(self, grains):  # pylint: disable=W0221
        return {git_pillar: {"__opts__": copy.copy(_OPTS), "__grains__": grains}}

    def get_pillar(self, ext_pillar_conf):
        """
        Run git_pillar with the specified configuration
        """
        cachedir = tempfile.mkdtemp(dir=RUNTIME_VARS.TMP)
        self.addCleanup(shutil.rmtree, cachedir, ignore_errors=True)
        ext_pillar_opts = {"optimization_order": [0, 1, 2]}
        ext_pillar_opts.update(
            salt.utils.yaml.safe_load(
                ext_pillar_conf.format(
                    cachedir=cachedir,
                    extmods=os.path.join(cachedir, "extmods"),
                    **self.ext_opts
                )
            )
        )
        with patch.dict(git_pillar.__opts__, ext_pillar_opts):
            return git_pillar.ext_pillar(
                "minion", {}, *ext_pillar_opts["ext_pillar"][0]["git"]
            )

    def make_repo(self, root_dir, user="root"):
        log.info("Creating test Git repo....")
        self.bare_repo = os.path.join(root_dir, "repo.git")
        self.bare_repo_backup = "{}.backup".format(self.bare_repo)
        self.admin_repo = os.path.join(root_dir, "admin")
        self.admin_repo_backup = "{}.backup".format(self.admin_repo)

        for dirname in (self.bare_repo, self.admin_repo):
            shutil.rmtree(dirname, ignore_errors=True)

        if os.path.exists(self.bare_repo_backup) and os.path.exists(
            self.admin_repo_backup
        ):
            shutil.copytree(self.bare_repo_backup, self.bare_repo)
            shutil.copytree(self.admin_repo_backup, self.admin_repo)
            return

        # Create bare repo
        self.run_function("git.init", [self.bare_repo], user=user, bare=True)

        # Clone bare repo
        self.run_function("git.clone", [self.admin_repo], url=self.bare_repo, user=user)

        def _push(branch, message):
            self.run_function("git.add", [self.admin_repo, "."], user=user)
            self.run_function(
                "git.commit",
                [self.admin_repo, message],
                user=user,
                git_opts=self.git_opts,
            )
            self.run_function(
                "git.push", [self.admin_repo], remote="origin", ref=branch, user=user,
            )

        with salt.utils.files.fopen(
            os.path.join(self.admin_repo, "top.sls"), "w"
        ) as fp_:
            fp_.write(
                textwrap.dedent(
                    """\
            base:
              '*':
                - foo
            """
                )
            )
        with salt.utils.files.fopen(
            os.path.join(self.admin_repo, "foo.sls"), "w"
        ) as fp_:
            fp_.write(
                textwrap.dedent(
                    """\
            branch: master
            mylist:
              - master
            mydict:
              master: True
              nested_list:
                - master
              nested_dict:
                master: True
            """
                )
            )
        # Add another file to be referenced using git_pillar_includes
        with salt.utils.files.fopen(
            os.path.join(self.admin_repo, "bar.sls"), "w"
        ) as fp_:
            fp_.write("included_pillar: True\n")
        # Add another file in subdir
        os.mkdir(os.path.join(self.admin_repo, "subdir"))
        with salt.utils.files.fopen(
            os.path.join(self.admin_repo, "subdir", "bar.sls"), "w"
        ) as fp_:
            fp_.write("from_subdir: True\n")
        _push("master", "initial commit")

        # Do the same with different values for "dev" branch
        self.run_function("git.checkout", [self.admin_repo], user=user, opts="-b dev")
        # The bar.sls shouldn't be in any branch but master
        self.run_function("git.rm", [self.admin_repo, "bar.sls"], user=user)
        with salt.utils.files.fopen(
            os.path.join(self.admin_repo, "top.sls"), "w"
        ) as fp_:
            fp_.write(
                textwrap.dedent(
                    """\
            dev:
              '*':
                - foo
            """
                )
            )
        with salt.utils.files.fopen(
            os.path.join(self.admin_repo, "foo.sls"), "w"
        ) as fp_:
            fp_.write(
                textwrap.dedent(
                    """\
            branch: dev
            mylist:
              - dev
            mydict:
              dev: True
              nested_list:
                - dev
              nested_dict:
                dev: True
            """
                )
            )
        _push("dev", "add dev branch")

        # Create just a top file in a separate repo, to be mapped to the base
        # env and referenced using git_pillar_includes
        self.run_function(
            "git.checkout", [self.admin_repo], user=user, opts="-b top_only"
        )
        # The top.sls should be the only file in this branch
        self.run_function(
            "git.rm",
            [self.admin_repo, "foo.sls", os.path.join("subdir", "bar.sls")],
            user=user,
        )
        with salt.utils.files.fopen(
            os.path.join(self.admin_repo, "top.sls"), "w"
        ) as fp_:
            fp_.write(
                textwrap.dedent(
                    """\
            base:
              '*':
                - bar
            """
                )
            )
        _push("top_only", "add top_only branch")

        # Create just another top file in a separate repo, to be mapped to the base
        # env and including mounted.bar
        self.run_function(
            "git.checkout", [self.admin_repo], user=user, opts="-b top_mounted"
        )
        # The top.sls should be the only file in this branch
        with salt.utils.files.fopen(
            os.path.join(self.admin_repo, "top.sls"), "w"
        ) as fp_:
            fp_.write(
                textwrap.dedent(
                    """\
            base:
              '*':
                - mounted.bar
            """
                )
            )
        _push("top_mounted", "add top_mounted branch")
        shutil.copytree(self.bare_repo, self.bare_repo_backup)
        shutil.copytree(self.admin_repo, self.admin_repo_backup)
        log.info("Test Git repo created.")

    def make_extra_repo(self, root_dir, user="root"):
        log.info("Creating extra test Git repo....")
        self.bare_extra_repo = os.path.join(root_dir, "extra_repo.git")
        self.bare_extra_repo_backup = "{}.backup".format(self.bare_extra_repo)
        self.admin_extra_repo = os.path.join(root_dir, "admin_extra")
        self.admin_extra_repo_backup = "{}.backup".format(self.admin_extra_repo)

        for dirname in (self.bare_extra_repo, self.admin_extra_repo):
            shutil.rmtree(dirname, ignore_errors=True)

        if os.path.exists(self.bare_extra_repo_backup) and os.path.exists(
            self.admin_extra_repo_backup
        ):
            shutil.copytree(self.bare_extra_repo_backup, self.bare_extra_repo)
            shutil.copytree(self.admin_extra_repo_backup, self.admin_extra_repo)
            return

        # Create bare extra repo
        self.run_function("git.init", [self.bare_extra_repo], user=user, bare=True)

        # Clone bare repo
        self.run_function(
            "git.clone", [self.admin_extra_repo], url=self.bare_extra_repo, user=user
        )

        def _push(branch, message):
            self.run_function("git.add", [self.admin_extra_repo, "."], user=user)
            self.run_function(
                "git.commit",
                [self.admin_extra_repo, message],
                user=user,
                git_opts=self.git_opts,
            )
            self.run_function(
                "git.push",
                [self.admin_extra_repo],
                remote="origin",
                ref=branch,
                user=user,
            )

        with salt.utils.files.fopen(
            os.path.join(self.admin_extra_repo, "top.sls"), "w"
        ) as fp_:
            fp_.write(
                textwrap.dedent(
                    """\
            "{{saltenv}}":
              '*':
                - motd
                - nowhere.foo
            """
                )
            )
        with salt.utils.files.fopen(
            os.path.join(self.admin_extra_repo, "motd.sls"), "w"
        ) as fp_:
            fp_.write(
                textwrap.dedent(
                    """\
            motd: The force will be with you. Always.
            """
                )
            )
        _push("master", "initial commit")
        shutil.copytree(self.bare_extra_repo, self.bare_extra_repo_backup)
        shutil.copytree(self.admin_extra_repo, self.admin_extra_repo_backup)
        log.info("Extra test Git repo created.")

    @classmethod
    def tearDownClass(cls):
        super(GitPillarTestBase, cls).tearDownClass()
        for dirname in (
            cls.admin_repo,
            cls.admin_repo_backup,
            cls.admin_extra_repo,
            cls.admin_extra_repo_backup,
            cls.bare_repo,
            cls.bare_repo_backup,
            cls.bare_extra_repo,
            cls.bare_extra_repo_backup,
        ):
            if dirname is not None:
                shutil.rmtree(dirname, ignore_errors=True)


class GitPillarSSHTestBase(GitPillarTestBase, SSHDMixin):
    """
    Base class for GitPython and Pygit2 SSH tests
    """

    id_rsa_nopass = id_rsa_withpass = None
    git_ssh = "/tmp/git_ssh"

    def setUp(self):
        """
        Create the SSH server and user, and create the git repo
        """
        log.info("%s.setUp() started...", self.__class__.__name__)
        super(GitPillarSSHTestBase, self).setUp()
        root_dir = os.path.expanduser("~{0}".format(self.username))
        if root_dir.startswith("~"):
            raise AssertionError(
                "Unable to resolve homedir for user '{0}'".format(self.username)
            )
        self.make_repo(root_dir, user=self.username)
        self.make_extra_repo(root_dir, user=self.username)
        log.info("%s.setUp() complete.", self.__class__.__name__)

    def get_pillar(self, ext_pillar_conf):
        """
        Wrap the parent class' get_pillar() func in logic that temporarily
        changes the GIT_SSH to use our custom script, ensuring that the
        passphraselsess key is used to auth without needing to modify the root
        user's ssh config file.
        """
        with patched_environ(GIT_SSH=self.git_ssh):
            return super(GitPillarSSHTestBase, self).get_pillar(ext_pillar_conf)


class GitPillarHTTPTestBase(GitPillarTestBase, WebserverMixin):
    """
    Base class for GitPython and Pygit2 HTTP tests
    """

    def setUp(self):
        """
        Create and start the webserver, and create the git repo
        """
        log.info("%s.setUp() started...", self.__class__.__name__)
        super(GitPillarHTTPTestBase, self).setUp()
        self.make_repo(self.repo_dir)
        self.make_extra_repo(self.repo_dir)
        log.info("%s.setUp() complete", self.__class__.__name__)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Tyler Johnson <tjohnson@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import

# Import Salt Libs
import salt.states.pkgrepo as pkgrepo

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class PkgrepoTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.pkgrepo
    """

    def setup_loader_modules(self):
        return {
            pkgrepo: {
                "__opts__": {"test": True},
                "__grains__": {"os": "", "os_family": ""},
            }
        }

    def test_new_key_url(self):
        """
        Test when only the key_url is changed that a change is triggered
        """
        kwargs = {
            "name": "deb http://mock/ sid main",
            "disabled": False,
        }
        key_url = "http://mock/changed_gpg.key"

        with patch.dict(
            pkgrepo.__salt__, {"pkg.get_repo": MagicMock(return_value=kwargs)}
        ):
            ret = pkgrepo.managed(key_url=key_url, **kwargs)
            self.assertDictEqual(
                {"key_url": {"old": None, "new": key_url}}, ret["changes"]
            )

    def test_update_key_url(self):
        """
        Test when only the key_url is changed that a change is triggered
        """
        kwargs = {
            "name": "deb http://mock/ sid main",
            "gpgcheck": 1,
            "disabled": False,
            "key_url": "http://mock/gpg.key",
        }
        changed_kwargs = kwargs.copy()
        changed_kwargs["key_url"] = "http://mock/gpg2.key"

        with patch.dict(
            pkgrepo.__salt__, {"pkg.get_repo": MagicMock(return_value=kwargs)}
        ):
            ret = pkgrepo.managed(**changed_kwargs)
            self.assertIn("key_url", ret["changes"], "Expected a change to key_url")
            self.assertDictEqual(
                {
                    "key_url": {
                        "old": kwargs["key_url"],
                        "new": changed_kwargs["key_url"],
                    }
                },
                ret["changes"],
            )



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import logging
import os
import shutil
import tempfile
import time

import salt.config
import salt.version
from tests.support.case import MultimasterModuleCase
from tests.support.helpers import slowTest
from tests.support.mixins import AdaptedConfigurationTestCaseMixin
from tests.support.unit import skipIf

try:
    import pyinotify  # pylint: disable=unused-import

    HAS_PYINOTIFY = True
except ImportError:
    HAS_PYINOTIFY = False


log = logging.getLogger(__name__)


@skipIf(not HAS_PYINOTIFY, "pyinotify is not available")
class TestBeaconsInotify(MultimasterModuleCase, AdaptedConfigurationTestCaseMixin):
    """
    Validate the inotify beacon in multimaster environment
    """

    def setUp(self):
        self.tmpdir = salt.utils.stringutils.to_unicode(tempfile.mkdtemp())
        self.addCleanup(shutil.rmtree, self.tmpdir, ignore_errors=True)

    @slowTest
    def test_beacons_duplicate_53344(self):
        # Also add a status beacon to use it for interval checks
        res = self.run_function(
            "beacons.add",
            ("inotify", [{"files": {self.tmpdir: {"mask": ["create"]}}}]),
            master_tgt="mm-master",
        )
        log.debug("Inotify beacon add returned: %s", res)
        self.assertTrue(res.get("result"))
        self.addCleanup(
            self.run_function, "beacons.delete", ("inotify",), master_tgt="mm-master"
        )
        res = self.run_function(
            "beacons.add", ("status", [{"time": ["all"]}]), master_tgt="mm-master",
        )
        log.debug("Status beacon add returned: %s", res)
        self.assertTrue(res.get("result"))
        self.addCleanup(
            self.run_function, "beacons.delete", ("status",), master_tgt="mm-master"
        )

        # Ensure beacons are added.
        res = self.run_function(
            "beacons.list", (), return_yaml=False, master_tgt="mm-master",
        )
        log.debug("Beacons list: %s", res)
        self.assertEqual(
            {
                "inotify": [{"files": {self.tmpdir: {"mask": ["create"]}}}],
                "status": [{"time": ["all"]}],
            },
            res,
        )

        file_path = os.path.join(self.tmpdir, "tmpfile")
        master_listener = salt.utils.event.get_master_event(
            self.mm_master_opts, sock_dir=self.mm_master_opts["sock_dir"]
        )
        self.addCleanup(master_listener.destroy)
        sub_master_listener = salt.utils.event.get_master_event(
            self.mm_sub_master_opts, sock_dir=self.mm_sub_master_opts["sock_dir"]
        )
        self.addCleanup(sub_master_listener.destroy)

        # We have to wait beacon first execution that would configure the inotify watch.
        # Since beacons will be executed both together waiting for the first status beacon event
        # which will mean the inotify beacon was executed too.
        start = time.time()
        stop_at = start + self.mm_minion_opts["loop_interval"] * 2 + 60
        event = sub_event = None
        while True:
            if time.time() > stop_at:
                break
            if not event:
                event = master_listener.get_event(
                    full=True,
                    wait=1,
                    tag="salt/beacon/mm-minion/status",
                    match_type="startswith",
                )
            if sub_event is None:
                sub_event = sub_master_listener.get_event(
                    full=True,
                    wait=1,
                    tag="salt/beacon/mm-minion/status",
                    match_type="startswith",
                )
            if event and sub_event:
                break
        log.debug("Status events received: %s, %s", event, sub_event)

        if not event or not sub_event:
            self.fail("Failed to receive at least one of the status events")

        with salt.utils.files.fopen(file_path, "w") as f:
            pass

        start = time.time()
        # Now in successful case this test will get results at most in 2 loop intervals.
        # Waiting for 2 loops intervals + some seconds to the hardware stupidity.
        stop_at = start + self.mm_minion_opts["loop_interval"] * 3 + 60
        event = sub_event = None
        expected_tag = salt.utils.stringutils.to_str(
            "salt/beacon/mm-minion/inotify/{}".format(self.tmpdir)
        )
        while True:
            if time.time() > stop_at:
                break
            if not event:
                event = master_listener.get_event(
                    full=True, wait=1, tag=expected_tag, match_type="startswith"
                )
            if sub_event is None:
                sub_event = sub_master_listener.get_event(
                    full=True, wait=1, tag=expected_tag, match_type="startswith"
                )
            if event and sub_event:
                break
        log.debug("Inotify events received: %s, %s", event, sub_event)

        if not event or not sub_event:
            self.fail("Failed to receive at least one of the inotify events")

        # We can't determine the timestamp so remove it from results
        if event:
            del event["data"]["_stamp"]
        if sub_event:
            del sub_event["data"]["_stamp"]

        expected = {
            "data": {"path": file_path, "change": "IN_CREATE", "id": "mm-minion"},
            "tag": expected_tag,
        }

        # It's better to compare both at once to see both responses in the error log.
        self.assertEqual((expected, expected), (event, sub_event))



# -*- coding: utf-8 -*-
"""
    :codeauthor: Pablo Suárez Hdez. <psuarezhernandez@suse.de>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.udev as udev
from salt.ext import six

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class UdevTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.udev
    """

    def setup_loader_modules(self):
        return {udev: {}}

    # 'info' function tests: 1

    def test_info(self):
        """
        Test if it returns the info of udev-created node in a dict
        """
        cmd_out = {
            "retcode": 0,
            "stdout": "P: /devices/virtual/vc/vcsa7\n"
            "N: vcsa7\n"
            "E: DEVNAME=/dev/vcsa7\n"
            "E: DEVPATH=/devices/virtual/vc/vcsa7\n"
            "E: MAJOR=7\n"
            "E: MINOR=135\n"
            "E: SUBSYSTEM=vc\n"
            "\n",
            "stderr": "",
        }

        ret = {
            "E": {
                "DEVNAME": "/dev/vcsa7",
                "DEVPATH": "/devices/virtual/vc/vcsa7",
                "MAJOR": 7,
                "MINOR": 135,
                "SUBSYSTEM": "vc",
            },
            "N": "vcsa7",
            "P": "/devices/virtual/vc/vcsa7",
        }

        mock = MagicMock(return_value=cmd_out)
        with patch.dict(udev.__salt__, {"cmd.run_all": mock}):
            data = udev.info("/dev/vcsa7")

            assert ret["P"] == data["P"]
            assert ret.get("N") == data.get("N")
            for key, value in six.iteritems(data["E"]):
                assert ret["E"][key] == value

    # 'exportdb' function tests: 1

    def test_exportdb(self):
        """
        Test if it returns the all the udev database into a dict
        """
        udev_data = """
P: /devices/LNXSYSTM:00/LNXPWRBN:00
E: DEVPATH=/devices/LNXSYSTM:00/LNXPWRBN:00
E: DRIVER=button
E: MODALIAS=acpi:LNXPWRBN:
E: SUBSYSTEM=acpi

P: /devices/LNXSYSTM:00/LNXPWRBN:00/input/input2
E: DEVPATH=/devices/LNXSYSTM:00/LNXPWRBN:00/input/input2
E: EV=3
E: ID_FOR_SEAT=input-acpi-LNXPWRBN_00
E: ID_INPUT=1
E: ID_INPUT_KEY=1
E: ID_PATH=acpi-LNXPWRBN:00
E: ID_PATH_TAG=acpi-LNXPWRBN_00
E: KEY=10000000000000 0
E: MODALIAS=input:b0019v0000p0001e0000-e0,1,k74,ramlsfw
E: NAME="Power Button"
E: PHYS="LNXPWRBN/button/input0"
E: PRODUCT=19/0/1/0
E: PROP=0
E: SUBSYSTEM=input
E: TAGS=:seat:
E: USEC_INITIALIZED=2010022

P: /devices/LNXSYSTM:00/LNXPWRBN:00/input/input2/event2
N: input/event2
E: BACKSPACE=guess
E: DEVNAME=/dev/input/event2
E: DEVPATH=/devices/LNXSYSTM:00/LNXPWRBN:00/input/input2/event2
E: ID_INPUT=1
E: ID_INPUT_KEY=1
E: ID_PATH=acpi-LNXPWRBN:00
E: ID_PATH_TAG=acpi-LNXPWRBN_00
E: MAJOR=13
E: MINOR=66
E: SUBSYSTEM=input
E: TAGS=:power-switch:
E: USEC_INITIALIZED=2076101
E: XKBLAYOUT=us
E: XKBMODEL=pc105
    """

        out = [
            {
                "P": "/devices/LNXSYSTM:00/LNXPWRBN:00",
                "E": {
                    "MODALIAS": "acpi:LNXPWRBN:",
                    "SUBSYSTEM": "acpi",
                    "DRIVER": "button",
                    "DEVPATH": "/devices/LNXSYSTM:00/LNXPWRBN:00",
                },
            },
            {
                "P": "/devices/LNXSYSTM:00/LNXPWRBN:00/input/input2",
                "E": {
                    "SUBSYSTEM": "input",
                    "PRODUCT": "19/0/1/0",
                    "PHYS": '"LNXPWRBN/button/input0"',
                    "NAME": '"Power Button"',
                    "ID_INPUT": 1,
                    "DEVPATH": "/devices/LNXSYSTM:00/LNXPWRBN:00/input/input2",
                    "MODALIAS": "input:b0019v0000p0001e0000-e0,1,k74,ramlsfw",
                    "ID_PATH_TAG": "acpi-LNXPWRBN_00",
                    "TAGS": ":seat:",
                    "PROP": 0,
                    "ID_FOR_SEAT": "input-acpi-LNXPWRBN_00",
                    "KEY": "10000000000000 0",
                    "USEC_INITIALIZED": 2010022,
                    "ID_PATH": "acpi-LNXPWRBN:00",
                    "EV": 3,
                    "ID_INPUT_KEY": 1,
                },
            },
            {
                "P": "/devices/LNXSYSTM:00/LNXPWRBN:00/input/input2/event2",
                "E": {
                    "SUBSYSTEM": "input",
                    "XKBLAYOUT": "us",
                    "MAJOR": 13,
                    "ID_INPUT": 1,
                    "DEVPATH": "/devices/LNXSYSTM:00/LNXPWRBN:00/input/input2/event2",
                    "ID_PATH_TAG": "acpi-LNXPWRBN_00",
                    "DEVNAME": "/dev/input/event2",
                    "TAGS": ":power-switch:",
                    "BACKSPACE": "guess",
                    "MINOR": 66,
                    "USEC_INITIALIZED": 2076101,
                    "ID_PATH": "acpi-LNXPWRBN:00",
                    "XKBMODEL": "pc105",
                    "ID_INPUT_KEY": 1,
                },
                "N": "input/event2",
            },
        ]

        mock = MagicMock(return_value={"retcode": 0, "stdout": udev_data})
        with patch.dict(udev.__salt__, {"cmd.run_all": mock}):
            data = udev.exportdb()
            assert data == [x for x in data if x]

            for d_idx, d_section in enumerate(data):
                assert out[d_idx]["P"] == d_section["P"]
                assert out[d_idx].get("N") == d_section.get("N")
                for key, value in six.iteritems(d_section["E"]):
                    assert out[d_idx]["E"][key] == value

    def test_normalize_info(self):
        """
        Test if udevdb._normalize_info does not returns nested lists that contains only one item.

        :return:
        """
        data = {"key": ["value", "here"], "foo": ["bar"], "some": "data"}
        assert udev._normalize_info(data) == {
            "foo": "bar",
            "some": "data",
            "key": ["value", "here"],
        }



# -*- coding: utf-8 -*-
"""
Support for nftables
"""
from __future__ import absolute_import, print_function, unicode_literals

# Import python libs
import json
import logging
import re

import salt.utils.data
import salt.utils.files
import salt.utils.path
from salt.exceptions import CommandExecutionError

# Import salt libs
from salt.ext import six
from salt.state import STATE_INTERNAL_KEYWORDS as _STATE_INTERNAL_KEYWORDS

# Set up logging
log = logging.getLogger(__name__)

_NFTABLES_FAMILIES = {
    "ipv4": "ip",
    "ip4": "ip",
    "ip": "ip",
    "ipv6": "ip6",
    "ip6": "ip6",
    "inet": "inet",
    "arp": "arp",
    "bridge": "bridge",
    "netdev": "netdev",
}


def __virtual__():
    """
    Only load the module if nftables is installed
    """
    if salt.utils.path.which("nft"):
        return "nftables"
    return (
        False,
        "The nftables execution module failed to load: nftables is not installed.",
    )


def _nftables_cmd():
    """
    Return correct command
    """
    return "nft"


def _conf(family="ip"):
    """
    Use the same file for rules for now.
    """
    if __grains__["os_family"] == "RedHat":
        return "/etc/nftables"
    elif __grains__["os_family"] == "Arch":
        return "/etc/nftables"
    elif __grains__["os_family"] == "Debian":
        return "/etc/nftables"
    elif __grains__["os"] == "Gentoo":
        return "/etc/nftables"
    else:
        return False


def version():
    """
    Return version from nftables --version

    CLI Example:

    .. code-block:: bash

        salt '*' nftables.version

    """
    cmd = "{0} --version".format(_nftables_cmd())
    out = __salt__["cmd.run"](cmd).split()
    return out[1]


def build_rule(
    table=None,
    chain=None,
    command=None,
    position="",
    full=None,
    family="ipv4",
    **kwargs
):
    """
    Build a well-formatted nftables rule based on kwargs.
    A `table` and `chain` are not required, unless `full` is True.

    If `full` is `True`, then `table`, `chain` and `command` are required.
    `command` may be specified as either insert, append, or delete.
    This will return the nftables command, exactly as it would
    be used from the command line.

    If a position is required (as with `insert` or `delete`), it may be specified as
    `position`. This will only be useful if `full` is True.

    If `connstate` is passed in, it will automatically be changed to `state`.

    CLI Examples:

    .. code-block:: bash

        salt '*' nftables.build_rule match=state \\
            connstate=RELATED,ESTABLISHED jump=ACCEPT
        salt '*' nftables.build_rule filter input command=insert position=3 \\
            full=True match=state state=related,established jump=accept

        IPv6:
        salt '*' nftables.build_rule match=state \\
            connstate=related,established jump=accept \\
            family=ipv6
        salt '*' nftables.build_rule filter input command=insert position=3 \\
            full=True match=state state=related,established jump=accept \\
            family=ipv6

    """
    ret = {"comment": "", "rule": "", "result": False}

    if "target" in kwargs:
        kwargs["jump"] = kwargs["target"]
        del kwargs["target"]

    for ignore in list(_STATE_INTERNAL_KEYWORDS) + ["chain", "save", "table"]:
        if ignore in kwargs:
            del kwargs[ignore]

    rule = ""
    proto = ""

    nft_family = _NFTABLES_FAMILIES[family]

    if "if" in kwargs:
        rule += "meta iifname {0} ".format(kwargs["if"])
        del kwargs["if"]

    if "of" in kwargs:
        rule += "meta oifname {0} ".format(kwargs["of"])
        del kwargs["of"]

    if "proto" in kwargs:
        proto = kwargs["proto"]

    if "state" in kwargs:
        del kwargs["state"]

    if "connstate" in kwargs:
        rule += "ct state {{ {0}}} ".format(kwargs["connstate"])
        del kwargs["connstate"]

    if "dport" in kwargs:
        kwargs["dport"] = six.text_type(kwargs["dport"])
        if ":" in kwargs["dport"]:
            kwargs["dport"] = kwargs["dport"].replace(":", "-")
        rule += "dport {{ {0} }} ".format(kwargs["dport"])
        del kwargs["dport"]

    if "sport" in kwargs:
        kwargs["sport"] = six.text_type(kwargs["sport"])
        if ":" in kwargs["sport"]:
            kwargs["sport"] = kwargs["sport"].replace(":", "-")
        rule += "sport {{ {0} }} ".format(kwargs["sport"])
        del kwargs["sport"]

    if "dports" in kwargs:
        # nftables reverse sorts the ports from
        # high to low, create rule like this
        # so that the check will work
        _dports = kwargs["dports"].split(",")
        _dports = [int(x) for x in _dports]
        _dports.sort(reverse=True)
        kwargs["dports"] = ", ".join(six.text_type(x) for x in _dports)

        rule += "dport {{ {0} }} ".format(kwargs["dports"])
        del kwargs["dports"]

    if "sports" in kwargs:
        # nftables reverse sorts the ports from
        # high to low, create rule like this
        # so that the check will work
        _sports = kwargs["sports"].split(",")
        _sports = [int(x) for x in _sports]
        _sports.sort(reverse=True)
        kwargs["sports"] = ", ".join(six.text_type(x) for x in _sports)

        rule += "sport {{ {0} }} ".format(kwargs["sports"])
        del kwargs["sports"]

    # Jumps should appear last, except for any arguments that are passed to
    # jumps, which of course need to follow.
    after_jump = []

    if "jump" in kwargs:
        after_jump.append("{0} ".format(kwargs["jump"]))
        del kwargs["jump"]

    if "j" in kwargs:
        after_jump.append("{0} ".format(kwargs["j"]))
        del kwargs["j"]

    if "to-port" in kwargs:
        after_jump.append("--to-port {0} ".format(kwargs["to-port"]))
        del kwargs["to-port"]

    if "to-ports" in kwargs:
        after_jump.append("--to-ports {0} ".format(kwargs["to-ports"]))
        del kwargs["to-ports"]

    if "to-destination" in kwargs:
        after_jump.append("--to-destination {0} ".format(kwargs["to-destination"]))
        del kwargs["to-destination"]

    if "reject-with" in kwargs:
        after_jump.append("--reject-with {0} ".format(kwargs["reject-with"]))
        del kwargs["reject-with"]

    for item in after_jump:
        rule += item

    # Strip trailing spaces off rule
    rule = rule.strip()

    # Insert the protocol prior to dport or sport
    rule = rule.replace("dport", "{0} dport".format(proto))
    rule = rule.replace("sport", "{0} sport".format(proto))

    ret["rule"] = rule

    if full in ["True", "true"]:

        if not table:
            ret["comment"] = "Table needs to be specified"
            return ret

        if not chain:
            ret["comment"] = "Chain needs to be specified"
            return ret

        if not command:
            ret["comment"] = "Command needs to be specified"
            return ret

        if command in ["Insert", "insert", "INSERT"]:
            if position:
                ret["rule"] = "{0} insert rule {1} {2} {3} " "position {4} {5}".format(
                    _nftables_cmd(), nft_family, table, chain, position, rule
                )
            else:
                ret["rule"] = "{0} insert rule " "{1} {2} {3} {4}".format(
                    _nftables_cmd(), nft_family, table, chain, rule
                )
        else:
            ret["rule"] = "{0} {1} rule {2} {3} {4} {5}".format(
                _nftables_cmd(), command, nft_family, table, chain, rule
            )

    if ret["rule"]:
        ret["comment"] = "Successfully built rule"
    ret["result"] = True
    return ret


def get_saved_rules(conf_file=None):
    """
    Return a data structure of the rules in the conf file

    CLI Example:

    .. code-block:: bash

        salt '*' nftables.get_saved_rules

    """
    if _conf() and not conf_file:
        conf_file = _conf()

    with salt.utils.files.fopen(conf_file) as fp_:
        lines = salt.utils.data.decode(fp_.readlines())
    rules = []
    for line in lines:
        tmpline = line.strip()
        if not tmpline:
            continue
        if tmpline.startswith("#"):
            continue
        rules.append(line)
    return rules


def list_tables(family="ipv4"):
    """
    Return a data structure of the current, in-memory tables

    CLI Example:

    .. code-block:: bash

        salt '*' nftables.list_tables

        salt '*' nftables.list_tables family=ipv6

    """
    nft_family = _NFTABLES_FAMILIES[family]
    tables = []
    cmd = "{0} --json --numeric --numeric --numeric " "list tables {1}".format(
        _nftables_cmd(), nft_family
    )
    out = __salt__["cmd.run"](cmd, python_shell=False)
    if not out:
        return tables
    data = json.loads(out)

    for item in data.get("nftables", []):
        if "metainfo" not in item:
            tables.append(item["table"])
    log.debug(tables)
    return tables


def get_rules(family="ipv4"):
    """
    Return a data structure of the current, in-memory rules

    CLI Example:

    .. code-block:: bash

        salt '*' nftables.get_rules

        salt '*' nftables.get_rules family=ipv6

    """
    tables = list_tables(family)
    nft_family = _NFTABLES_FAMILIES[family]

    rules = []
    for table in tables:
        table_name = table["name"]
        cmd = "{0} --numeric --numeric --numeric " "list table {1} {2}".format(
            _nftables_cmd(), nft_family, table_name
        )
        out = __salt__["cmd.run"](cmd, python_shell=False)
        rules.append(out)
    return rules


def save(filename=None, family="ipv4"):
    """
    Save the current in-memory rules to disk

    CLI Example:

    .. code-block:: bash

        salt '*' nftables.save /etc/nftables
    """
    if _conf() and not filename:
        filename = _conf()

    nft_families = ["ip", "ip6", "arp", "bridge"]
    rules = "#! nft -f\n"
    for family in nft_families:
        out = get_rules(family)
        if out:
            rules += "\n"
        rules = rules + "\n".join(out)
    rules = rules + "\n"

    try:
        with salt.utils.files.fopen(filename, "wb") as _fh:
            # Write out any changes
            _fh.write(salt.utils.data.encode(rules))
    except (IOError, OSError) as exc:
        raise CommandExecutionError(
            "Problem writing to configuration file: {0}".format(exc)
        )
    return rules


def get_rule_handle(table="filter", chain=None, rule=None, family="ipv4"):
    """
    Get the handle for a particular rule

    This function accepts a rule in a standard nftables command format,
        starting with the chain. Trying to force users to adapt to a new
        method of creating rules would be irritating at best, and we
        already have a parser that can handle it.

    CLI Example:

    .. code-block:: bash

        salt '*' nftables.get_rule_handle filter input \\
            rule='tcp dport 22 log accept'

        IPv6:
        salt '*' nftables.get_rule_handle filter input \\
            rule='tcp dport 22 log accept' \\
            family=ipv6
    """
    ret = {"comment": "", "result": False}

    if not chain:
        ret["comment"] = "Chain needs to be specified"
        return ret

    if not rule:
        ret["comment"] = "Rule needs to be specified"
        return ret

    res = check_table(table, family=family)
    if not res["result"]:
        return res

    res = check_chain(table, chain, family=family)
    if not res["result"]:
        return res

    res = check(table, chain, rule, family=family)
    if not res["result"]:
        return res

    nft_family = _NFTABLES_FAMILIES[family]
    cmd = "{0} --numeric --numeric --numeric --handle list chain {1} {2} {3}".format(
        _nftables_cmd(), nft_family, table, chain
    )
    out = __salt__["cmd.run"](cmd, python_shell=False)
    rules = re.split("\n+", out)

    pat = re.compile(r"{0} # handle (?P<handle>\d+)".format(rule))
    for r in rules:
        match = pat.search(r)
        if match:
            return {"result": True, "handle": match.group("handle")}
    return {"result": False, "comment": "Could not find rule {0}".format(rule)}


def check(table="filter", chain=None, rule=None, family="ipv4"):
    """
    Check for the existence of a rule in the table and chain

    This function accepts a rule in a standard nftables command format,
        starting with the chain. Trying to force users to adapt to a new
        method of creating rules would be irritating at best, and we
        already have a parser that can handle it.

    CLI Example:

    .. code-block:: bash

        salt '*' nftables.check filter input \\
            rule='tcp dport 22 log accept'

        IPv6:
        salt '*' nftables.check filter input \\
            rule='tcp dport 22 log accept' \\
            family=ipv6
    """
    ret = {"comment": "", "result": False}

    if not chain:
        ret["comment"] = "Chain needs to be specified"
        return ret

    if not rule:
        ret["comment"] = "Rule needs to be specified"
        return ret

    res = check_table(table, family=family)
    if not res["result"]:
        return res

    res = check_chain(table, chain, family=family)
    if not res["result"]:
        return res

    nft_family = _NFTABLES_FAMILIES[family]
    cmd = "{0} --handle --numeric --numeric --numeric list chain {1} {2} {3}".format(
        _nftables_cmd(), nft_family, table, chain
    )
    search_rule = "{0} #".format(rule)
    out = __salt__["cmd.run"](cmd, python_shell=False).find(search_rule)

    if out == -1:
        ret[
            "comment"
        ] = "Rule {0} in chain {1} in table {2} in family {3} does not exist".format(
            rule, chain, table, family
        )
    else:
        ret[
            "comment"
        ] = "Rule {0} in chain {1} in table {2} in family {3} exists".format(
            rule, chain, table, family
        )
        ret["result"] = True
    return ret


def check_chain(table="filter", chain=None, family="ipv4"):
    """
    .. versionadded:: 2014.7.0

    Check for the existence of a chain in the table

    CLI Example:

    .. code-block:: bash

        salt '*' nftables.check_chain filter input

        IPv6:
        salt '*' nftables.check_chain filter input family=ipv6
    """

    ret = {"comment": "", "result": False}

    if not chain:
        ret["comment"] = "Chain needs to be specified"
        return ret

    nft_family = _NFTABLES_FAMILIES[family]
    cmd = "{0} list table {1} {2}".format(_nftables_cmd(), nft_family, table)
    out = __salt__["cmd.run"](cmd, python_shell=False).find(
        "chain {0} {{".format(chain)
    )

    if out == -1:
        ret["comment"] = "Chain {0} in table {1} in family {2} does not exist".format(
            chain, table, family
        )
    else:
        ret["comment"] = "Chain {0} in table {1} in family {2} exists".format(
            chain, table, family
        )
        ret["result"] = True
    return ret


def check_table(table=None, family="ipv4"):
    """
    Check for the existence of a table

    CLI Example::

        salt '*' nftables.check_table nat
    """
    ret = {"comment": "", "result": False}

    if not table:
        ret["comment"] = "Table needs to be specified"
        return ret

    nft_family = _NFTABLES_FAMILIES[family]
    cmd = "{0} list tables {1}".format(_nftables_cmd(), nft_family)
    out = __salt__["cmd.run"](cmd, python_shell=False).find(
        "table {0} {1}".format(nft_family, table)
    )

    if out == -1:
        ret["comment"] = "Table {0} in family {1} does not exist".format(table, family)
    else:
        ret["comment"] = "Table {0} in family {1} exists".format(table, family)
        ret["result"] = True
    return ret


def new_table(table, family="ipv4"):
    """
    .. versionadded:: 2014.7.0

    Create new custom table.

    CLI Example:

    .. code-block:: bash

        salt '*' nftables.new_table filter

        IPv6:
        salt '*' nftables.new_table filter family=ipv6
    """
    ret = {"comment": "", "result": False}

    if not table:
        ret["comment"] = "Table needs to be specified"
        return ret

    res = check_table(table, family=family)
    if res["result"]:
        return res

    nft_family = _NFTABLES_FAMILIES[family]
    cmd = "{0} add table {1} {2}".format(_nftables_cmd(), nft_family, table)
    out = __salt__["cmd.run"](cmd, python_shell=False)

    if not out:
        ret["comment"] = "Table {0} in family {1} created".format(table, family)
        ret["result"] = True
    else:
        ret["comment"] = "Table {0} in family {1} could not be created".format(
            table, family
        )
    return ret


def delete_table(table, family="ipv4"):
    """
    .. versionadded:: 2014.7.0

    Create new custom table.

    CLI Example:

    .. code-block:: bash

        salt '*' nftables.delete_table filter

        IPv6:
        salt '*' nftables.delete_table filter family=ipv6
    """
    ret = {"comment": "", "result": False}

    if not table:
        ret["comment"] = "Table needs to be specified"
        return ret

    res = check_table(table, family=family)
    if not res["result"]:
        return res

    nft_family = _NFTABLES_FAMILIES[family]
    cmd = "{0} delete table {1} {2}".format(_nftables_cmd(), nft_family, table)
    out = __salt__["cmd.run"](cmd, python_shell=False)

    if not out:
        ret["comment"] = "Table {0} in family {1} deleted".format(table, family)
        ret["result"] = True
    else:
        ret["comment"] = "Table {0} in family {1} could not be deleted".format(
            table, family
        )
    return ret


def new_chain(
    table="filter", chain=None, table_type=None, hook=None, priority=None, family="ipv4"
):
    """
    .. versionadded:: 2014.7.0

    Create new chain to the specified table.

    CLI Example:

    .. code-block:: bash

        salt '*' nftables.new_chain filter input

        salt '*' nftables.new_chain filter input \\
                table_type=filter hook=input priority=0

        salt '*' nftables.new_chain filter foo

        IPv6:
        salt '*' nftables.new_chain filter input family=ipv6

        salt '*' nftables.new_chain filter input \\
                table_type=filter hook=input priority=0 family=ipv6

        salt '*' nftables.new_chain filter foo family=ipv6
    """
    ret = {"comment": "", "result": False}

    if not chain:
        ret["comment"] = "Chain needs to be specified"
        return ret

    res = check_table(table, family=family)
    if not res["result"]:
        return res

    res = check_chain(table, chain, family=family)
    if res["result"]:
        ret["comment"] = "Chain {0} in table {1} in family {2} already exists".format(
            chain, table, family
        )
        return ret

    nft_family = _NFTABLES_FAMILIES[family]
    cmd = "{0} add chain {1} {2} {3}".format(_nftables_cmd(), nft_family, table, chain)
    if table_type or hook or priority:
        if table_type and hook and six.text_type(priority):
            cmd = r"{0} \{{ type {1} hook {2} priority {3}\; \}}".format(
                cmd, table_type, hook, priority
            )
        else:
            # Specify one, require all
            ret["comment"] = "Table_type, hook, and priority required."
            return ret

    out = __salt__["cmd.run"](cmd, python_shell=False)

    if not out:
        ret["comment"] = "Chain {0} in table {1} in family {2} created".format(
            chain, table, family
        )
        ret["result"] = True
    else:
        ret[
            "comment"
        ] = "Chain {0} in table {1} in family {2} could not be created".format(
            chain, table, family
        )
    return ret


def delete_chain(table="filter", chain=None, family="ipv4"):
    """
    .. versionadded:: 2014.7.0

    Delete the chain from the specified table.

    CLI Example:

    .. code-block:: bash

        salt '*' nftables.delete_chain filter input

        salt '*' nftables.delete_chain filter foo

        IPv6:
        salt '*' nftables.delete_chain filter input family=ipv6

        salt '*' nftables.delete_chain filter foo family=ipv6
    """
    ret = {"comment": "", "result": False}

    if not chain:
        ret["comment"] = "Chain needs to be specified"
        return ret

    res = check_table(table, family=family)
    if not res["result"]:
        return res

    res = check_chain(table, chain, family=family)
    if not res["result"]:
        return res

    nft_family = _NFTABLES_FAMILIES[family]
    cmd = "{0} delete chain {1} {2} {3}".format(
        _nftables_cmd(), nft_family, table, chain
    )
    out = __salt__["cmd.run"](cmd, python_shell=False)

    if not out:
        ret["comment"] = "Chain {0} in table {1} in family {2} deleted".format(
            chain, table, family
        )
        ret["result"] = True
    else:
        ret[
            "comment"
        ] = "Chain {0} in table {1} in family {2} could not be deleted".format(
            chain, table, family
        )
    return ret


def append(table="filter", chain=None, rule=None, family="ipv4"):
    """
    Append a rule to the specified table & chain.

    This function accepts a rule in a standard nftables command format,
        starting with the chain. Trying to force users to adapt to a new
        method of creating rules would be irritating at best, and we
        already have a parser that can handle it.

    CLI Example:

    .. code-block:: bash

        salt '*' nftables.append filter input \\
            rule='tcp dport 22 log accept'

        IPv6:
        salt '*' nftables.append filter input \\
            rule='tcp dport 22 log accept' \\
            family=ipv6
    """
    ret = {
        "comment": "Failed to append rule {0} to chain {1} in table {2}.".format(
            rule, chain, table
        ),
        "result": False,
    }

    if not chain:
        ret["comment"] = "Chain needs to be specified"
        return ret

    if not rule:
        ret["comment"] = "Rule needs to be specified"
        return ret

    res = check_table(table, family=family)
    if not res["result"]:
        return res

    res = check_chain(table, chain, family=family)
    if not res["result"]:
        return res

    res = check(table, chain, rule, family=family)
    if res["result"]:
        ret[
            "comment"
        ] = "Rule {0} chain {1} in table {2} in family {3} already exists".format(
            rule, chain, table, family
        )
        return ret

    nft_family = _NFTABLES_FAMILIES[family]
    cmd = "{0} add rule {1} {2} {3} {4}".format(
        _nftables_cmd(), nft_family, table, chain, rule
    )
    out = __salt__["cmd.run"](cmd, python_shell=False)

    if len(out) == 0:
        ret["result"] = True
        ret[
            "comment"
        ] = 'Added rule "{0}" chain {1} in table {2} in family {3}.'.format(
            rule, chain, table, family
        )
    else:
        ret[
            "comment"
        ] = 'Failed to add rule "{0}" chain {1} in table {2} in family {3}.'.format(
            rule, chain, table, family
        )
    return ret


def insert(table="filter", chain=None, position=None, rule=None, family="ipv4"):
    """
    Insert a rule into the specified table & chain, at the specified position.

    If position is not specified, rule will be inserted in first position.

    This function accepts a rule in a standard nftables command format,
        starting with the chain. Trying to force users to adapt to a new
        method of creating rules would be irritating at best, and we
        already have a parser that can handle it.

    CLI Examples:

    .. code-block:: bash

        salt '*' nftables.insert filter input \\
            rule='tcp dport 22 log accept'

        salt '*' nftables.insert filter input position=3 \\
            rule='tcp dport 22 log accept'

        IPv6:
        salt '*' nftables.insert filter input \\
            rule='tcp dport 22 log accept' \\
            family=ipv6

        salt '*' nftables.insert filter input position=3 \\
            rule='tcp dport 22 log accept' \\
            family=ipv6
    """
    ret = {
        "comment": "Failed to insert rule {0} to table {1}.".format(rule, table),
        "result": False,
    }

    if not chain:
        ret["comment"] = "Chain needs to be specified"
        return ret

    if not rule:
        ret["comment"] = "Rule needs to be specified"
        return ret

    res = check_table(table, family=family)
    if not res["result"]:
        return res

    res = check_chain(table, chain, family=family)
    if not res["result"]:
        return res

    res = check(table, chain, rule, family=family)
    if res["result"]:
        ret[
            "comment"
        ] = "Rule {0} chain {1} in table {2} in family {3} already exists".format(
            rule, chain, table, family
        )
        return ret

    nft_family = _NFTABLES_FAMILIES[family]
    if position:
        cmd = "{0} insert rule {1} {2} {3} position {4} {5}".format(
            _nftables_cmd(), nft_family, table, chain, position, rule
        )
    else:
        cmd = "{0} insert rule {1} {2} {3} {4}".format(
            _nftables_cmd(), nft_family, table, chain, rule
        )
    out = __salt__["cmd.run"](cmd, python_shell=False)

    if len(out) == 0:
        ret["result"] = True
        ret[
            "comment"
        ] = 'Added rule "{0}" chain {1} in table {2} in family {3}.'.format(
            rule, chain, table, family
        )
    else:
        ret[
            "comment"
        ] = 'Failed to add rule "{0}" chain {1} in table {2} in family {3}.'.format(
            rule, chain, table, family
        )
    return ret


def delete(table, chain=None, position=None, rule=None, family="ipv4"):
    """
    Delete a rule from the specified table & chain, specifying either the rule
        in its entirety, or the rule's position in the chain.

    This function accepts a rule in a standard nftables command format,
        starting with the chain. Trying to force users to adapt to a new
        method of creating rules would be irritating at best, and we
        already have a parser that can handle it.

    CLI Examples:

    .. code-block:: bash

        salt '*' nftables.delete filter input position=3

        salt '*' nftables.delete filter input \\
            rule='tcp dport 22 log accept'

        IPv6:
        salt '*' nftables.delete filter input position=3 family=ipv6

        salt '*' nftables.delete filter input \\
            rule='tcp dport 22 log accept' \\
            family=ipv6
    """
    ret = {
        "comment": "Failed to delete rule {0} in table {1}.".format(rule, table),
        "result": False,
    }

    if position and rule:
        ret["comment"] = "Only specify a position or a rule, not both"
        return ret

    res = check_table(table, family=family)
    if not res["result"]:
        return res

    res = check_chain(table, chain, family=family)
    if not res["result"]:
        return res

    res = check(table, chain, rule, family=family)
    if not res["result"]:
        ret[
            "comment"
        ] = "Rule {0} chain {1} in table {2} in family {3} does not exist".format(
            rule, chain, table, family
        )
        return ret

    # nftables rules can only be deleted using the handle
    # if we don't have it, find it.
    if not position:
        position = get_rule_handle(table, chain, rule, family)

    nft_family = _NFTABLES_FAMILIES[family]
    cmd = "{0} delete rule {1} {2} {3} handle {4}".format(
        _nftables_cmd(), nft_family, table, chain, position
    )
    out = __salt__["cmd.run"](cmd, python_shell=False)

    if len(out) == 0:
        ret["result"] = True
        ret[
            "comment"
        ] = 'Deleted rule "{0}" in chain {1} in table {2} in family {3}.'.format(
            rule, chain, table, family
        )
    else:
        ret[
            "comment"
        ] = 'Failed to delete rule "{0}" in chain {1}  table {2} in family {3}'.format(
            rule, chain, table, family
        )
    return ret


def flush(table="filter", chain="", family="ipv4"):
    """
    Flush the chain in the specified table, flush all chains in the specified
    table if chain is not specified.

    CLI Example:

    .. code-block:: bash

        salt '*' nftables.flush filter

        salt '*' nftables.flush filter input

        IPv6:
        salt '*' nftables.flush filter input family=ipv6
    """
    ret = {
        "comment": "Failed to flush rules from chain {0} in table {1}.".format(
            chain, table
        ),
        "result": False,
    }

    res = check_table(table, family=family)
    if not res["result"]:
        return res

    nft_family = _NFTABLES_FAMILIES[family]

    if chain:
        res = check_chain(table, chain, family=family)
        if not res["result"]:
            return res
        cmd = "{0} flush chain {1} {2} {3}".format(
            _nftables_cmd(), nft_family, table, chain
        )
        comment = "from chain {0} in table {1} in family {2}.".format(
            chain, table, family
        )
    else:
        cmd = "{0} flush table {1} {2}".format(_nftables_cmd(), nft_family, table)
        comment = "from table {0} in family {1}.".format(table, family)
    out = __salt__["cmd.run"](cmd, python_shell=False)

    if len(out) == 0:
        ret["result"] = True
        ret["comment"] = "Flushed rules {0}".format(comment)
    else:
        ret["comment"] = "Failed to flush rules {0}".format(comment)
    return ret



# -*- coding: utf-8 -*-
"""
integration tests for mac_pkgutil
"""
from __future__ import absolute_import, print_function, unicode_literals

import os

from tests.support.case import ModuleCase
from tests.support.helpers import (
    destructiveTest,
    requires_system_grains,
    runs_on,
    skip_if_binaries_missing,
    skip_if_not_root,
    slowTest,
)
from tests.support.runtests import RUNTIME_VARS

TEST_PKG_URL = (
    "https://distfiles.macports.org/MacPorts/MacPorts-2.3.4-10.11-ElCapitan.pkg"
)
TEST_PKG_NAME = "org.macports.MacPorts"


@runs_on(kernel="Darwin")
@skip_if_not_root
@skip_if_binaries_missing("pkgutil")
class MacPkgutilModuleTest(ModuleCase):
    """
    Validate the mac_pkgutil module
    """

    @classmethod
    def setUpClass(cls):
        cls.test_pkg = os.path.join(
            RUNTIME_VARS.TMP, "MacPorts-2.3.4-10.11-ElCapitan.pkg"
        )

    @requires_system_grains
    def setUp(self, grains):  # pylint: disable=arguments-differ
        """
        Get current settings
        """
        os_release = grains["osrelease"]
        self.pkg_name = "com.apple.pkg.BaseSystemResources"
        if int(os_release.split(".")[1]) >= 13:
            self.pkg_name = "com.apple.pkg.iTunesX"

    def tearDown(self):
        """
        Reset to original settings
        """
        self.run_function("pkgutil.forget", [TEST_PKG_NAME])
        self.run_function("file.remove", ["/opt/local"])

    @slowTest
    def test_list(self):
        """
        Test pkgutil.list
        """
        self.assertIsInstance(self.run_function("pkgutil.list"), list)
        self.assertIn(self.pkg_name, self.run_function("pkgutil.list"))

    @slowTest
    def test_is_installed(self):
        """
        Test pkgutil.is_installed
        """
        # Test Package is installed
        self.assertTrue(self.run_function("pkgutil.is_installed", [self.pkg_name]))

        # Test Package is not installed
        self.assertFalse(self.run_function("pkgutil.is_installed", ["spongebob"]))

    @destructiveTest
    @slowTest
    def test_install_forget(self):
        """
        Test pkgutil.install
        Test pkgutil.forget
        """
        # Test if installed
        self.assertFalse(self.run_function("pkgutil.is_installed", [TEST_PKG_NAME]))

        # Download the package
        self.run_function("cp.get_url", [TEST_PKG_URL, self.test_pkg])

        # Test install
        self.assertTrue(
            self.run_function("pkgutil.install", [self.test_pkg, TEST_PKG_NAME])
        )
        self.assertIn(
            "Unsupported scheme",
            self.run_function("pkgutil.install", ["ftp://test", "spongebob"]),
        )

        # Test forget
        self.assertTrue(self.run_function("pkgutil.forget", [TEST_PKG_NAME]))



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.apache_module as apache_module

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class ApacheModuleTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.apache_module
    """

    def setup_loader_modules(self):
        return {apache_module: {}}

    # 'enabled' function tests: 1

    def test_enabled(self):
        """
        Test to ensure an Apache module is enabled.
        """
        name = "cgi"

        ret = {"name": name, "result": True, "changes": {}, "comment": ""}

        mock = MagicMock(side_effect=[True, False, False])
        mock_str = MagicMock(return_value={"Status": ["enabled"]})
        with patch.dict(
            apache_module.__salt__,
            {"apache.check_mod_enabled": mock, "apache.a2enmod": mock_str},
        ):
            comt = "{0} already enabled.".format(name)
            ret.update({"comment": comt})
            self.assertDictEqual(apache_module.enabled(name), ret)

            comt = "Apache module {0} is set to be enabled.".format(name)
            ret.update(
                {
                    "comment": comt,
                    "result": None,
                    "changes": {"new": "cgi", "old": None},
                }
            )
            with patch.dict(apache_module.__opts__, {"test": True}):
                self.assertDictEqual(apache_module.enabled(name), ret)

            comt = "Failed to enable {0} Apache module".format(name)
            ret.update({"comment": comt, "result": False, "changes": {}})
            with patch.dict(apache_module.__opts__, {"test": False}):
                self.assertDictEqual(apache_module.enabled(name), ret)

    # 'disabled' function tests: 1

    def test_disabled(self):
        """
        Test to ensure an Apache module is disabled.
        """
        name = "cgi"

        ret = {"name": name, "result": None, "changes": {}, "comment": ""}

        mock = MagicMock(side_effect=[True, True, False])
        mock_str = MagicMock(return_value={"Status": ["disabled"]})
        with patch.dict(
            apache_module.__salt__,
            {"apache.check_mod_enabled": mock, "apache.a2dismod": mock_str},
        ):
            comt = "Apache module {0} is set to be disabled.".format(name)
            ret.update({"comment": comt, "changes": {"new": None, "old": "cgi"}})
            with patch.dict(apache_module.__opts__, {"test": True}):
                self.assertDictEqual(apache_module.disabled(name), ret)

            comt = "Failed to disable {0} Apache module".format(name)
            ret.update({"comment": comt, "result": False, "changes": {}})
            with patch.dict(apache_module.__opts__, {"test": False}):
                self.assertDictEqual(apache_module.disabled(name), ret)

            comt = "{0} already disabled.".format(name)
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(apache_module.disabled(name), ret)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.boto_sns as boto_sns

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class BotoSnsTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.boto_sns
    """

    def setup_loader_modules(self):
        return {boto_sns: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to ensure the SNS topic exists.
        """
        name = "test.example.com."

        ret = {"name": name, "result": True, "changes": {}, "comment": ""}

        mock = MagicMock(side_effect=[True, False, False])
        mock_bool = MagicMock(return_value=False)
        with patch.dict(
            boto_sns.__salt__, {"boto_sns.exists": mock, "boto_sns.create": mock_bool}
        ):
            comt = "AWS SNS topic {0} present.".format(name)
            ret.update({"comment": comt})
            self.assertDictEqual(boto_sns.present(name), ret)

            with patch.dict(boto_sns.__opts__, {"test": True}):
                comt = "AWS SNS topic {0} is set to be created.".format(name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(boto_sns.present(name), ret)

            with patch.dict(boto_sns.__opts__, {"test": False}):
                comt = "Failed to create {0} AWS SNS topic".format(name)
                ret.update({"comment": comt, "result": False})
                self.assertDictEqual(boto_sns.present(name), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to ensure the named sns topic is deleted.
        """
        name = "test.example.com."

        ret = {"name": name, "result": True, "changes": {}, "comment": ""}

        self.maxDiff = None

        exists_mock = MagicMock(side_effect=[False, True, True, True, True, True, True])
        with patch.dict(boto_sns.__salt__, {"boto_sns.exists": exists_mock}):
            # tests topic already absent
            comt = "AWS SNS topic {0} does not exist.".format(name)
            ret.update({"comment": comt})
            self.assertDictEqual(boto_sns.absent(name), ret)

            with patch.dict(boto_sns.__opts__, {"test": True}):
                # tests topic present, test option, unsubscribe is False
                comt = (
                    "AWS SNS topic {0} is set to be removed.  "
                    "0 subscription(s) will be removed.".format(name)
                )
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(boto_sns.absent(name), ret)

            subscriptions = [
                dict(
                    Endpoint="arn:aws:lambda:us-west-2:123456789:function:test",
                    Owner=123456789,
                    Protocol="Lambda",
                    TopicArn="arn:aws:sns:us-west-2:123456789:test",
                    SubscriptionArn="arn:aws:sns:us-west-2:123456789:test:some_uuid",
                )
            ]
            with patch.dict(boto_sns.__opts__, {"test": True}):
                subs_mock = MagicMock(return_value=subscriptions)
                with patch.dict(
                    boto_sns.__salt__,
                    {"boto_sns.get_all_subscriptions_by_topic": subs_mock},
                ):
                    # tests topic present, 1 subscription, test option, unsubscribe is True
                    comt = (
                        "AWS SNS topic {0} is set to be removed.  "
                        "1 subscription(s) will be removed.".format(name)
                    )
                    ret.update({"comment": comt, "result": None})
                    self.assertDictEqual(boto_sns.absent(name, unsubscribe=True), ret)

            subs_mock = MagicMock(return_value=subscriptions)
            unsubscribe_mock = MagicMock(side_effect=[True, False])
            with patch.dict(
                boto_sns.__salt__, {"boto_sns.unsubscribe": unsubscribe_mock}
            ):
                with patch.dict(
                    boto_sns.__salt__,
                    {"boto_sns.get_all_subscriptions_by_topic": subs_mock},
                ):
                    delete_mock = MagicMock(side_effect=[True, True, True, False])
                    with patch.dict(
                        boto_sns.__salt__, {"boto_sns.delete": delete_mock}
                    ):
                        # tests topic present, unsubscribe flag True, unsubscribe succeeded,
                        # delete succeeded
                        comt = "AWS SNS topic {0} deleted.".format(name)
                        ret.update(
                            {
                                "changes": {
                                    "new": None,
                                    "old": {
                                        "topic": name,
                                        "subscriptions": subscriptions,
                                    },
                                },
                                "result": True,
                                "comment": comt,
                            }
                        )
                        self.assertDictEqual(
                            boto_sns.absent(name, unsubscribe=True), ret
                        )

                        # tests topic present, unsubscribe flag True, unsubscribe fails,
                        # delete succeeded
                        ret.update(
                            {
                                "changes": {
                                    "new": {"subscriptions": subscriptions},
                                    "old": {
                                        "topic": name,
                                        "subscriptions": subscriptions,
                                    },
                                },
                                "result": True,
                                "comment": comt,
                            }
                        )
                        self.assertDictEqual(
                            boto_sns.absent(name, unsubscribe=True), ret
                        )

                        # tests topic present, unsubscribe flag False, delete succeeded
                        ret.update(
                            {
                                "changes": {"new": None, "old": {"topic": name}},
                                "result": True,
                                "comment": comt,
                            }
                        )
                        self.assertDictEqual(boto_sns.absent(name), ret)

                        # tests topic present, unsubscribe flag False, delete failed
                        comt = "Failed to delete {0} AWS SNS topic.".format(name)
                        ret.update({"changes": {}, "result": False, "comment": comt})
                        self.assertDictEqual(boto_sns.absent(name), ret)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import os

import salt.config
import salt.loader

# Import Salt Libs
import salt.states.reg as reg
import salt.utils.platform
import salt.utils.win_reg
from tests.support.helpers import destructiveTest

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import patch
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import TestCase, skipIf


@skipIf(not salt.utils.platform.is_windows(), "System is not Windows")
class RegTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.reg
    """

    hive = "HKEY_CURRENT_USER"
    key = "SOFTWARE\\Salt-Testing"
    name = hive + "\\" + key
    vname = "version"
    vdata = "0.15.3"

    def setup_loader_modules(self):
        opts = salt.config.minion_config(
            os.path.join(RUNTIME_VARS.TMP_CONF_DIR, "minion")
        )
        utils = salt.loader.utils(opts, whitelist=["dacl", "reg"])
        return {reg: {"__opts__": {"test": False}, "__utils__": utils}}

    def tearDown(self):
        salt.utils.win_reg.delete_key_recursive(hive=self.hive, key=self.key)

    @destructiveTest
    def test_present(self):
        """
        Test to set a registry entry.
        """
        expected = {
            "comment": "Added {0} to {1}".format(self.vname, self.name),
            "changes": {
                "reg": {
                    "Added": {
                        "Inheritance": True,
                        "Perms": {"Deny": None, "Grant": None},
                        "Value": self.vdata,
                        "Key": self.name,
                        "Owner": None,
                        "Entry": self.vname,
                    }
                }
            },
            "name": self.name,
            "result": True,
        }
        ret = reg.present(self.name, vname=self.vname, vdata=self.vdata)
        self.assertDictEqual(ret, expected)

    @destructiveTest
    def test_present_string_dword(self):
        """
        Test to set a registry entry.
        """
        vname = "dword_data"
        vdata = "00000001"
        vtype = "REG_DWORD"
        expected_vdata = 1
        expected = {
            "comment": "Added {0} to {1}".format(vname, self.name),
            "changes": {
                "reg": {
                    "Added": {
                        "Inheritance": True,
                        "Perms": {"Deny": None, "Grant": None},
                        "Value": expected_vdata,
                        "Key": self.name,
                        "Owner": None,
                        "Entry": vname,
                    }
                }
            },
            "name": self.name,
            "result": True,
        }
        ret = reg.present(self.name, vname=vname, vdata=vdata, vtype=vtype)
        self.assertDictEqual(ret, expected)

    @destructiveTest
    def test_present_string_dword_existing(self):
        """
        Test to set a registry entry.
        """
        vname = "dword_data"
        vdata = "0000001"
        vtype = "REG_DWORD"
        # Set it first
        reg.present(self.name, vname=vname, vdata=vdata, vtype=vtype)
        expected = {
            "comment": "{0} in {1} is already present".format(vname, self.name),
            "changes": {},
            "name": self.name,
            "result": True,
        }
        ret = reg.present(self.name, vname=vname, vdata=vdata, vtype=vtype)
        self.assertDictEqual(ret, expected)

    def test_present_test_true(self):
        expected = {
            "comment": "",
            "changes": {
                "reg": {
                    "Will add": {
                        "Inheritance": True,
                        "Perms": {"Deny": None, "Grant": None},
                        "Value": self.vdata,
                        "Key": self.name,
                        "Owner": None,
                        "Entry": "version",
                    }
                }
            },
            "name": self.name,
            "result": None,
        }
        with patch.dict(reg.__opts__, {"test": True}):
            ret = reg.present(self.name, vname=self.vname, vdata=self.vdata)
        self.assertDictEqual(ret, expected)

    def test_present_existing(self):
        # Create the reg key for testing
        salt.utils.win_reg.set_value(
            hive=self.hive, key=self.key, vname=self.vname, vdata=self.vdata
        )

        expected = {
            "comment": "{0} in {1} is already present".format(self.vname, self.name),
            "changes": {},
            "name": self.name,
            "result": True,
        }
        ret = reg.present(self.name, vname=self.vname, vdata=self.vdata)
        self.assertDictEqual(ret, expected)

    def test_present_existing_key_only(self):
        """
        Test setting only a key with no value name
        """
        # Create the reg key for testing
        salt.utils.win_reg.set_value(hive=self.hive, key=self.key)

        expected = {
            "comment": "(Default) in {0} is already present".format(self.name),
            "changes": {},
            "name": self.name,
            "result": True,
        }
        ret = reg.present(self.name)
        self.assertDictEqual(ret, expected)

    def test_present_existing_test_true(self):
        # Create the reg key for testing
        salt.utils.win_reg.set_value(
            hive=self.hive, key=self.key, vname=self.vname, vdata=self.vdata
        )

        expected = {
            "comment": "{0} in {1} is already present".format(self.vname, self.name),
            "changes": {},
            "name": self.name,
            "result": True,
        }
        with patch.dict(reg.__opts__, {"test": True}):
            ret = reg.present(self.name, vname=self.vname, vdata=self.vdata)
        self.assertDictEqual(ret, expected)

    @destructiveTest
    def test_absent(self):
        """
        Test to remove a registry entry.
        """
        # Create the reg key for testing
        salt.utils.win_reg.set_value(
            hive=self.hive, key=self.key, vname=self.vname, vdata=self.vdata
        )
        expected = {
            "comment": "Removed {0} from {1}".format(self.key, self.hive),
            "changes": {"reg": {"Removed": {"Entry": self.vname, "Key": self.name}}},
            "name": self.name,
            "result": True,
        }
        ret = reg.absent(self.name, self.vname)
        self.assertDictEqual(ret, expected)

    @destructiveTest
    def test_absent_test_true(self):
        # Create the reg key for testing
        salt.utils.win_reg.set_value(
            hive=self.hive, key=self.key, vname=self.vname, vdata=self.vdata
        )
        expected = {
            "comment": "",
            "changes": {
                "reg": {"Will remove": {"Entry": self.vname, "Key": self.name}}
            },
            "name": self.name,
            "result": None,
        }
        with patch.dict(reg.__opts__, {"test": True}):
            ret = reg.absent(self.name, self.vname)
        self.assertDictEqual(ret, expected)

    def test_absent_already_absent(self):
        """
        Test to remove a registry entry.
        """
        expected = {
            "comment": "{0} is already absent".format(self.name),
            "changes": {},
            "name": self.name,
            "result": True,
        }
        ret = reg.absent(self.name, self.vname)
        self.assertDictEqual(ret, expected)

    def test_absent_already_absent_test_true(self):
        """
        Test to remove a registry entry.
        """
        expected = {
            "comment": "{0} is already absent".format(self.name),
            "changes": {},
            "name": self.name,
            "result": True,
        }
        with patch.dict(reg.__opts__, {"test": True}):
            ret = reg.absent(self.name, self.vname)
        self.assertDictEqual(ret, expected)
